import types
from cStringIO import StringIO

from ..base import MarshalBase, WritableContext, EMPTY_CONTEXT, ZERO_MIN_MAX
from ..errors import StructNotWellDefinedError

def copy_if_supported(obj):
    return obj.copy() if hasattr(obj, 'copy') else obj

class FieldMarshal(MarshalBase):
    def prepare_class(self, cls):
        pass
    
    def prepare_instance(self, obj, args, kwargs):
        pass
    
    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        raise NotImplementedError()
    
    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        raise NotImplementedError()

class FieldBase(FieldMarshal):
    def __init__(self, marshal):
        super(FieldBase, self).__init__()
        self.marshal = marshal

    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        self.marshal.write_to_stream(self.get_value(obj), stream, context)

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        self.set_value(obj, self.marshal.create_from_stream(stream, context, *args, **kwargs))
        
    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return self.marshal.to_repr(self.get_value(obj), context)

    def sizeof(self, obj):
        return self.marshal.sizeof(self.get_value(obj))

    def min_max_sizeof(self):
        return self.marshal.min_max_sizeof()

    def set_value(self, obj, value):
        pass

    def get_value(self, obj):
        return None

class AnonymousField(FieldBase):
    def __init__(self, marshal):
        super(AnonymousField, self).__init__(marshal)

    def __str__(self):
        return "anonymous field"

class Field(FieldBase):
    def __init__(self, name, marshal, default=None):
        super(Field, self).__init__(marshal)
        self.name = name
        self.default = default
    
    def prepare_instance(self, obj, args, kwargs):
        if self.name in kwargs:
            setattr(obj, self.name, kwargs.pop(self.name))
        elif not hasattr(obj, self.name):
            setattr(obj, self.name, copy_if_supported(self.default))
    
    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return "%s=%s" % (self.name, super(Field, self).to_repr(obj, context))

    def get_value(self, obj):
        return getattr(obj, self.name, self.default)

    def set_value(self, obj, value):
        setattr(obj, self.name, value)

    def __str__(self):
        return "field %s" % self.name

class FieldListContainer(FieldMarshal):
    def __init__(self, fields):
        super(FieldListContainer, self).__init__()
        self.fields = fields

    def prepare_class(self, cls):
        for field in self.fields:
            field.prepare_class(cls)

    def prepare_instance(self, obj, args, kwargs):
        for field in self.fields:
            field.prepare_instance(obj, args, kwargs)

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        for field in self.fields:
            field.read_fields(obj, stream, context, *args, **kwargs)

    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        for field in self.fields:
            field.write_fields(obj, stream, context)

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return ", ".join([ field.to_repr(obj, context) for field in self.fields ])

    def sizeof(self, obj):
        return sum([ field.sizeof(obj) for field in self.fields ])

    def min_max_sizeof(self):
        return sum([ field.min_max_sizeof() for field in self.fields ], ZERO_MIN_MAX)

class StructType(type):
    def __new__(cls, name, bases, attrs):
        # If we initialize our own class don't do any modifications.
        if name == "Struct":
            return super(StructType, cls).__new__(cls, name, bases, attrs)

        # We want to first put our own __init__ method that will initialize all the fields passed by kwargs and then
        # call the user's __init__ method (if exists) with args/kwargs left.
        if "__init__" in attrs:
            user_init = attrs["__init__"]
            del attrs["__init__"]
        else:
            user_init = None

        new_cls = super(StructType, cls).__new__(cls, name, bases, attrs)

        # We wait for the class to be created to check the fields, because we want to take advantage of Python's MRO
        # to resolve the _fields_ structure.
        if not hasattr(new_cls, "_fields_"):
            raise StructNotWellDefinedError("Class %s is missing a _fields_ declaration" % (name,))

        fields = getattr(new_cls, "_fields_")
        
        if isinstance(fields, (types.ListType, types.TupleType)):
            fields = FieldListContainer(fields)

        setattr(new_cls, "_container_", fields)

        fields.prepare_class(new_cls)

        setattr(new_cls, "__init__", cls._create_struct_class_init(new_cls, user_init))
        return new_cls

    @classmethod
    def _create_struct_class_init(cls, new_cls, user_init):
        def __instance_init__(self, *args, **kwargs):
            orig_kwargs = kwargs.copy()
            if type(self) == new_cls:
                # Only do our magic if we're the bottom-most class.
                new_cls._container_.prepare_instance(self, args, kwargs)
            if user_init is None:
                super(new_cls, self).__init__(*args, **kwargs)
            else:
                # Add arguments back if the user code wants them.
                # This solves STORAGEMODEL-146, because ASI defines Read10Command(logical_block_address, ...)
                # and logical_block_address is also a field in the struct, so in this scenario instruct will
                # "eat" the arg and we need to put it back.
                for arg in user_init.func_code.co_varnames[0:user_init.func_code.co_argcount]:
                    if arg in orig_kwargs:
                        kwargs[arg] = orig_kwargs[arg]
                user_init(self, *args, **kwargs)
        return __instance_init__

class Struct(object):
    __metaclass__ = StructType

    def __init__(self, *args, **kwargs):
        super(Struct, self).__init__()
        type(self)._container_.prepare_instance(self, args, kwargs)
        
    @classmethod
    def write_to_stream(cls, obj, stream, context=EMPTY_CONTEXT):
        context = cls.get_updated_context(obj, context)
        type(obj)._container_.write_fields(obj, stream, context)

    @classmethod
    def create_from_stream(cls, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        obj = cls(*args, **kwargs)
        context = cls.get_updated_context(obj, context)
        cls._container_.read_fields(obj, stream, context, *args, **kwargs)
        return obj

    @classmethod
    def min_max_sizeof(cls):
        return cls._container_.min_max_sizeof()
    
    @classmethod
    def sizeof(cls, obj):
        return cls._container_.sizeof(obj)

    @classmethod
    def write_to_string(cls, obj, context=EMPTY_CONTEXT):
        stream = StringIO()
        type(obj).write_to_stream(obj, stream, context)
        result = stream.getvalue()
        stream.close()
        return result

    @classmethod
    def create_from_string(cls, str, context=EMPTY_CONTEXT, *args, **kwargs):
        stream = StringIO(str)
        obj = cls.create_from_stream(stream, context, *args, **kwargs)
        stream.close()
        return obj

    @classmethod
    def to_repr(cls, obj, context=EMPTY_CONTEXT):
        if obj is None:
            return "<none>"
        cls = type(obj)
        context = cls.get_updated_context(obj, context)
        return("%s(%s)" % (cls.__name__, cls._container_.to_repr(obj, context)))

    @classmethod
    def get_updated_context(cls, obj, context):
        cls = type(obj)
        if hasattr(cls, '_context_'):
            new_context = WritableContext(cls._context_.copy())
        else:
            new_context = WritableContext(dict(struct=obj))
        new_context.update(context.get_dict())
        return new_context

    def __str__(self):
        return type(self).write_to_string(self)

    def __repr__(self):
        return type(self).to_repr(self)



from .base import FixedSizer, Marshal, EMPTY_CONTEXT

class FixedSizeMappingMarshal(FixedSizer, Marshal):
    def __init__(self, value_map, value_marshal):
        super(FixedSizeMappingMarshal, self).__init__()
        assert value_marshal.is_fixed_size()
        self.value_map = value_map
        self.value_marshal = value_marshal
        self.reverse_value_map = dict()
        self.size = self.value_marshal.min_max_sizeof().min
        for key, value in value_map.items():
            assert value not in self.reverse_value_map
            self.reverse_value_map[value] = key
        
    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        value = self.value_marshal.create_from_stream(stream, context, *args, **kwargs)
        return self.reverse_value_map[value]

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        value = self.value_map[obj]
        self.value_marshal.write_to_stream(value, stream, context)



__version__ = "0.6.11"
__git_commiter_name__ = "Tal Yalon"
__git_commiter_email__ = "yalon@infinidat.com"
__git_branch__ = '(Not currently on any branch)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '2362168df63f6d6270eb53a778e034fa5efeec23'
__git_head_subject__ = "- added default field value by using default= - added kwargs to buffer's init so you can initialize fields in buffer from the ctor (MyBuffer(my_field=1))"
__git_head_message__ = ''
__git_dirty_diff__ = ''


import traceback
import itertools

def format_exception(exc_info, indent=""):
    line_groups = [ line.strip("\n") for line in traceback.format_exception(*exc_info) ]
    lines = itertools.chain(*[ lines.split("\n") for lines in line_groups ])
    return "\n".join([ "{0}{1}".format(indent, line) for line in lines ])



import math
import struct

from .io_buffer import BitAwareByteArray
from .buffer import BufferType

from ..errors import InstructError
from ..utils.kwargs import (copy_defaults_and_override_with_kwargs, assert_kwarg_enum, copy_and_remove_kwargs,
                            keep_kwargs_partial)


def kwargs_fractional_byte_size(kwargs):
    byte_size = kwargs.get("byte_size", None)
    assert byte_size is None or ((byte_size > 0) and (int(byte_size * 8) == byte_size * 8)), \
        "size can be unknown (None) or a positive fraction of 8 but instead got {0}".format(byte_size)
    return byte_size


def kwargs_int_byte_size(kwargs):
    byte_size = kwargs.get("byte_size", None)
    assert byte_size is None or ((byte_size > 0) and (int(byte_size) == byte_size)), \
        "size can be unknown (None) or a positive integer but instead got {0}".format(byte_size)
    return byte_size


def assert_enum_argument(name, value, enum):
    assert value in enum, "argument {0} with value {1!r} is not one of {2!r}".format(name, value, enum)


class PackError(InstructError):
    MESSAGE = "{pack_ref!r} failed to pack {value!r}."

    def __init__(self, pack_ref, value):
        super(PackError, self).__init__(PackError.MESSAGE.format(pack_ref=pack_ref, value=value))


class UnpackError(InstructError):
    MESSAGE = "{unpack_ref!r} failed to unpack {buffer!r}."

    def __init__(self, unpack_ref, buffer):
        super(UnpackError, self).__init__(UnpackError.MESSAGE.format(unpack_ref=unpack_ref, buffer=buffer))


ENDIAN_NAME_TO_FORMAT = {'unspecified': '@', 'native': '=', 'big': '>', 'little': '<'}


#
# integer support
#


def pack_bit_int(value, byte_size, **kwargs):
    assert byte_size is not None
    result = BitAwareByteArray(bytearray(int(math.ceil(byte_size))), 0, byte_size)
    result[0:] = value
    return result


def format_from_struct_int_arguments(format_char, kwargs):
    args = copy_defaults_and_override_with_kwargs(dict(sign='signed', endian='native'), kwargs)
    assert_enum_argument('format_char', format_char, ('b', 'h', 'l', 'q'))
    assert_kwarg_enum(args, 'sign', ('signed', 'unsigned'))
    assert_kwarg_enum(args, 'endian', ENDIAN_NAME_TO_FORMAT.keys())
    format_char = format_char.lower() if args["sign"] == "signed" else format_char.upper()
    return "{0}{1}".format(ENDIAN_NAME_TO_FORMAT[args["endian"]], format_char)


def pack_struct_int(value, format_char, **kwargs):
    return struct.pack(format_from_struct_int_arguments(format_char, kwargs), value)

STRUCT_INT_PACKERS = {
    1: keep_kwargs_partial(pack_struct_int, format_char='b'),
    2: keep_kwargs_partial(pack_struct_int, format_char='h'),
    4: keep_kwargs_partial(pack_struct_int, format_char='l'),
    8: keep_kwargs_partial(pack_struct_int, format_char='q')
}


def pack_int(value, **kwargs):
    byte_size = kwargs_fractional_byte_size(kwargs)
    if byte_size in STRUCT_INT_PACKERS:
        return STRUCT_INT_PACKERS[byte_size](value, **kwargs)
    else:
        byte_size = kwargs.pop("byte_size", float(value.bit_length()) / 8)
        return pack_bit_int(value, byte_size, **kwargs)


def unpack_bit_int(buffer, byte_size, **kwargs):
    result = 0
    l = reversed(buffer[0:byte_size]) if kwargs.get("endian", "big") else buffer[0:byte_size]
    for b in l:
        result *= 256
        result += b
    return result, byte_size


def unpack_struct_int(buffer, format_char, **kwargs):
    format = format_from_struct_int_arguments(format_char, kwargs)
    byte_size = struct.calcsize(format)
    assert len(buffer) >= byte_size, "buffer size must be at least {0} but instead got {1}".format(byte_size, len(buffer))
    return struct.unpack(format, str(buffer[0:byte_size]))[0], byte_size

STRUCT_INT_UNPACKERS = {
    1: keep_kwargs_partial(unpack_struct_int, format_char='b'),
    2: keep_kwargs_partial(unpack_struct_int, format_char='h'),
    4: keep_kwargs_partial(unpack_struct_int, format_char='l'),
    8: keep_kwargs_partial(unpack_struct_int, format_char='q')
}


def unpack_int(buffer, **kwargs):
    byte_size = kwargs_fractional_byte_size(kwargs)
    if byte_size in STRUCT_INT_UNPACKERS:
        return STRUCT_INT_UNPACKERS[byte_size](buffer, **kwargs)
    else:
        byte_size = kwargs.pop('byte_size', buffer.length())
        return unpack_bit_int(buffer, byte_size, **kwargs)

#
# float support
#


def format_from_struct_float_arguments(format_char, kwargs):
    args = copy_defaults_and_override_with_kwargs(dict(endian='native'), kwargs)
    assert_enum_argument('format_char', format_char, ('f', 'd'))
    assert_kwarg_enum(args, 'endian', ENDIAN_NAME_TO_FORMAT.keys())
    return "{0}{1}".format(ENDIAN_NAME_TO_FORMAT[args["endian"]], format_char)


def pack_struct_float(value, format_char, **kwargs):
    return struct.pack(format_from_struct_float_arguments(format_char, kwargs), value)


def pack_float(value, **kwargs):
    byte_size = kwargs_int_byte_size(kwargs)
    if byte_size is None:
        byte_size = 4  # by default, we'll choose floats. If someone wants something different, he/she should define it.
    assert byte_size in STRUCT_FLOAT_PACKERS, "float must have a byte size of 4 or 8 but instead got {0}".format(byte_size)
    return STRUCT_FLOAT_PACKERS[byte_size](value, **kwargs)


STRUCT_FLOAT_PACKERS = {
    4: keep_kwargs_partial(pack_struct_float, format_char='f'),
    8: keep_kwargs_partial(pack_struct_float, format_char='q')
}


def unpack_struct_float(buffer, format_char, **kwargs):
    format = format_from_struct_float_arguments(format_char, kwargs)
    byte_size = struct.calcsize(format)
    assert len(buffer) >= byte_size, "buffer size must be at least {0} but instead got {1}".format(byte_size, len(buffer))
    return struct.unpack(format, str(buffer[0:byte_size]))[0], byte_size


def unpack_float(buffer, **kwargs):
    byte_size = kwargs_int_byte_size(kwargs)
    if byte_size is None:
        byte_size = 4  # by default, we'll choose floats. If someone wants something different, he/she should define it.
    assert byte_size in STRUCT_FLOAT_UNPACKERS, "float must have a byte size of 4 or 8 but instead got {0}".format(byte_size)
    return STRUCT_FLOAT_UNPACKERS[byte_size](buffer, **kwargs)

STRUCT_FLOAT_UNPACKERS = {
    4: keep_kwargs_partial(unpack_struct_float, format_char='f'),
    8: keep_kwargs_partial(unpack_struct_float, format_char='q')
}


#
# string support
#
JUSTIFY_OPTIONS = ('left', 'right', 'center')


def str_args_from_kwargs(kwargs):
    args = copy_defaults_and_override_with_kwargs(dict(encoding='ascii', padding=' ', justify='left'), kwargs)
    assert_kwarg_enum(args, 'justify', JUSTIFY_OPTIONS)
    assert len(args['padding']) == 1, "padding must be exactly one character but instead got {0!r}".format(args['padding'])
    return args


def pack_str(value, **kwargs):
    args = str_args_from_kwargs(kwargs)
    justify = args['justify']
    padding = args['padding']
    byte_size = kwargs_int_byte_size(args)
    result = bytearray(str(value).encode(args['encoding']))
    if byte_size is not None:
        if justify == 'left':
            result = result.ljust(byte_size, padding)
        elif justify == 'right':
            result = result.rjust(byte_size, padding)
        else:  # center
            result = result.center(byte_size, padding)
    return result


def unpack_str(buffer, **kwargs):
    args = str_args_from_kwargs(kwargs)
    justify = args['justify']
    padding = args['padding']
    byte_size = kwargs_int_byte_size(args)
    value = str(buffer[0:byte_size]).decode(args['encoding'])
    if byte_size is not None:
        if justify == 'left':
            value = value.rstrip(padding)
        elif justify == 'right':
            value = value.lstrip(padding)
        else:  # center
            value = value.strip(padding)
    return value, len(buffer)


def pack_bytearray(buffer, **kwargs):
    return bytearray(buffer)


def unpack_bytearray(buffer, **kwargs):
    return bytearray(buffer), len(buffer)


#
# buffers support
#


def pack_buffer(value, **kwargs):
    return value.pack()


def unpack_buffer(buffer, **kwargs):
    obj = kwargs['type']()
    byte_size = obj.unpack(buffer)
    return obj, byte_size


def unpack_selector_decorator(selector):
    def my_selector(obj):
        o = selector(obj)
        if isinstance(o, BufferType):
            result = o()
            byte_size = result.unpack(buffer)
            return result, byte_size
        elif isinstance(o, (list, tuple)):
            assert len(o) == 2, "selector returned a list but not a (obj, byte_size)-kind of list"
            return o
        elif o is None:
            return None, 0
        else:
            assert False, "selector didn't return a Buffer type, a pair of (obj, byte_size) or None, instead it returned {0!r}".format(o)


#
# lists support
#


def pack_list(list, elem_packer, **kwargs):
    result = BitAwareByteArray(bytearray())
    for o in list:
        result += elem_packer(o, **kwargs)
    return result


def unpack_list(buffer, elem_unpacker, **kwargs):
    n = kwargs.get('n', None)
    assert n is None or isinstance(n, (int, long)), "n must be either an integer or None but instead got {0!r}".format(n)
    byte_size = kwargs_fractional_byte_size(kwargs)

    result = []
    offset = 0
    index = 0
    while offset < buffer.length() and (n is None or index < n):
        unpacker_kwargs = copy_and_remove_kwargs(kwargs, ('buffer', 'n', 'index', 'container'))
        item, item_len = elem_unpacker(buffer[offset:byte_size], index=index, n=n, container=result, **unpacker_kwargs)
        result.append(item)
        offset += item_len
        index += 1
    return result, offset



class StructTypeAdapter(object):
    def __init__(self, buffer_type):
        self.buffer_type = buffer_type

    def create_from_string(self, s):
        buf = self.buffer_type()
        buf.unpack(s)
        return buf

    def write_to_stream(self, *args):
        raise NotImplementedError()

    def create_from_stream(self, *args, **kwargs):
        raise NotImplementedError()

    def min_max_sizeof(self):
        raise NotImplementedError()

    def sizeof(self, obj):
        raise NotImplementedError()

    def write_to_string(self, *args, **kwargs):
        raise NotImplementedError()

    def to_repr(self, *args, **kwargs):
        raise NotImplementedError()

    def get_updated_context(self, *args, **kwargs):
        raise NotImplementedError()


def buffer_to_struct_adapter(buffer_type):
    return StructTypeAdapter(buffer_type)



from ..base import Marshal, MinMax, ZERO_MIN_MAX, UNBOUNDED_MIN_MAX, EMPTY_CONTEXT
from ..errors import InstructError
from ..utils.read_ahead_stream import ReadAheadStream

from . import Struct

class StructSelectorMarshal(Marshal):
    def __init__(self, key_marshal, mapping, default=None):
        super(StructSelectorMarshal, self).__init__()
        self.key_marshal = key_marshal
        self.mapping = {}
        self.default = default
        
        for key, struct in self.mapping:
            if not isinstance(struct, Struct):
                raise ValueError("mapping key %s must be a subclass of Struct" % key)
            self.mapping[key] = struct

        structs = mapping.values() + ([ self.default ] if self.default is not None else [])

        self.min_max_size = sum([ struct.min_max_sizeof() for struct in structs ], ZERO_MIN_MAX)

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        type(obj).write_to_stream(obj, stream, context)
        
    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        rstream = ReadAheadStream(stream)
        
        rstream.set_read_ahead(True)
        key = self.key_marshal.create_from_stream(rstream, context)
        rstream.set_read_ahead(False)
        if key in self.mapping:
            marshal = self.mapping[key]
        else:
            if self.default is not None:
                marshal = self.default
            else:
                raise InstructError("key %s is not mapped an no default" % (self.key_marshal.to_repr(key, context),))

        result = marshal.create_from_stream(rstream, context, *args, **kwargs)
        if not rstream.is_read_ahead_empty():
            raise InstructError("deserialized key %s with type %s but still have bytes in read-ahead buffer" %
                                (self.key_marshal.to_repr(key, context), marshal))
        return result
    
    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return type(obj).to_repr(obj, context)
    
    def sizeof(self, obj):
        return type(obj).sizeof(obj)

    def min_max_sizeof(self):
        return self.min_max_size

class FuncStructSelectorMarshal(Marshal):
    def __init__(self, func, min_max_size=UNBOUNDED_MIN_MAX):
        super(FuncStructSelectorMarshal, self).__init__()
        self.func = func
        self.min_max_size = MinMax(min_max_size)

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        type(obj).write_to_stream(obj, stream, context)
        
    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        rstream = ReadAheadStream(stream)
        
        rstream.set_read_ahead(True)
        
        reader = self.func(rstream, context)

        rstream.set_read_ahead(False)

        result = reader.create_from_stream(rstream, context, *args, **kwargs)
        if not rstream.is_read_ahead_empty():
            raise InstructError("deserialized object %s but still have bytes in read-ahead buffer" %
                                (result.to_repr(context),))
        return result
    
    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return type(obj).to_repr(obj, context)
    
    def sizeof(self, obj):
        return type(obj).sizeof(obj)

    def min_max_sizeof(self):
        return self.min_max_size



import sys
import types
import collections
from cStringIO import StringIO

class ReadOnlyContext(object):
    """
    Context is a holder of properties that can modify and aid the marshalling process. It is used internally by Instruct
    to communicate additional information between marshalling components and can be used by the user to modify the
    marshalling behavior.
    """
    def __init__(self, d={}):
        self.d = d
        
    def get(self, key, default):
        return self.d.get(key, default)

    def get_dict(self):
        return self.d.copy()

    def writable_copy(self, dict={}):
        c = self.d.copy()
        c.update(dict)
        return WritableContext(c)

    def __repr__(self):
        return "ReadOnlyContext(%s)" % self._shallow_repr()

    def _shallow_repr(self):
        def _is_primitive(obj):
            return isinstance(obj, (types.IntType, types.BooleanType, types.BufferType, types.FloatType, types.ListType,
                                    types.TupleType) + types.StringTypes)
        return ", ".join([ "%s=%s" % (key, repr(val) if _is_primitive(val) else "%s<%s>" % (type(val), id(val)))
                           for key, val in self.d.items() ])

class WritableContext(ReadOnlyContext):
    def __init__(self, d=None):
        self.d = d or {}
    
    def put(self, key, value):
        self.d[key] = value

    def update(self, dict):
        self.d.update(dict)

    def remove(self, key):
        del self.d[key]

    def __repr__(self):
        return "WritableContext(%s)" % self._shallow_repr()

EMPTY_CONTEXT = ReadOnlyContext()

class MinMax(collections.namedtuple('MinMax', 'min max')):
    """
    MinMax is a simple object containing a minimum value and a maximum value. Values are clipped to be [0, sys.maxint].
    You can access the min/max values by the *min* or *max* properties or by index ([0], [1]).
    """
    __slots__ = ()
    
    def __new__(cls, min_val_or_tuple=0, max_val=sys.maxint):
        """
        Initialize a new MinMax instance. This initializer works in four different modes:
         * Copy constructor. If you pass it a MinMax object it'll copy the min/max values.
         * Construct from tuple/list. Construct the min/max by accessing [0] and [1].
         * Construct from a single argument. User provides the min value, max is set to sys.maxint.
         * Construct from two arguments. User provides the min and max values.
        """
        if isinstance(min_val_or_tuple, (list, tuple, MinMax)):
            assert len(min_val_or_tuple) == 2
            min_val, max_val = min_val_or_tuple
        else:
            min_val = min_val_or_tuple
        
        assert min_val <= max_val
        return super(MinMax, cls).__new__(cls, max(0, min_val), min(max_val, sys.maxint))

    def __add__(self, min_max):
        return MinMax(min_max.min + self.min, min_max.max + self.max)

    def is_unbounded(self):
        return self.max >= sys.maxint

    def __str__(self):
        return "MinMax(min=%d, max=%s)" % (self.min, self.max if self.max < sys.maxint else "unbounded")

    def __repr__(self):
        return str(self)

UNBOUNDED_MIN_MAX = MinMax(0, sys.maxint)
ZERO_MIN_MAX = MinMax(0, 0)

class MarshalBase(object):
    def sizeof(self, obj):
        raise NotImplementedError()
    
    # The following methods _should_ be implemented by the user:
    #
    def min_max_sizeof(self):
        return UNBOUNDED_MIN_MAX

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return repr(obj)

    # The following methods are convenience methods and generally don't need to get overridden:
    #
    def is_fixed_size(self):
        min_max = self.min_max_sizeof()
        return min_max.min == min_max.max

class Marshal(MarshalBase):
    # The following methods _must_ be implemented by the user.
    #
    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        raise NotImplementedError()

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        raise NotImplementedError()

    def sizeof(self, obj):
        raise NotImplementedError()

    # The following methods are convenience methods and generally don't need to get overridden:
    #
    def create_from_string(self, string, context=EMPTY_CONTEXT, *args, **kwargs):
        """
        Deserializes a new instance from a string.
        This is a convenience method that creates a StringIO object and calls create_instance_from_stream().
        """
        io = StringIO(string)
        instance = self.create_from_stream(io, context, *args, **kwargs)
        io.close()
        return instance

    def write_to_string(self, obj, context=EMPTY_CONTEXT):
        io = StringIO()
        instance = self.write_to_stream(obj, io, context)
        result = io.getvalue()
        io.close()
        return result

class FixedSizer(object):
    def sizeof(self, obj):
        return self.size

    def min_max_sizeof(self):
        return MinMax(self.size, self.size)

class ConstReader(FixedSizer, Marshal):
    """
    A marshal that doesn't write to a stream but returns a constant value whenever create_from_stream is called.
    """
    def __init__(self, value):
        self.value = value
        self.size = 0

    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        return self.value

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        pass

class CallableReader(FixedSizer, Marshal):
    """
    A marshal that doesn't write to a stream but returns a value from a callable whenever create_from_stream is called.
    """
    def __init__(self, func):
        self.func = func
        self.size = 0

    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        return self.func(stream, context)

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        pass



from infi.exceptools import InfiException

class InstructError(InfiException):
    pass

class NotEnoughDataError(InstructError):
    def __init__(self, expected, actually_read):
        super(NotEnoughDataError, self).__init__("expected to read %d bytes but read only %d bytes instead" %
                                                 (expected, actually_read))

class StructNotWellDefinedError(InstructError):
    pass

class BitFieldNotInByteBoundry(InstructError):
    pass

class FieldTypeNotSupportedError(InstructError):
    pass

class InvalidValueError(InstructError):
    pass

class ValidationValueIsNoneError(InstructError):
    def __init__(self):
        super(ValidationValueIsNoneError, self).__init__("Value cannot be None")



class BitStringIO(object):
    """
    Naive bit stream implementation.
    """
    def __init__(self, byte_size_or_array):
        self.position = 0
        self.value = bytearray(byte_size_or_array)

    def getvalue(self):
        return str(self.value)

    def write(self, value, bits_to_write):
        if (bits_to_write + self.position) > len(self.value) * 8:
            raise IOError("attempting to write past the end of the buffer")

        byte_i, bit_i = (self.position / 8, self.position % 8)

        bits_written = 0
        if bit_i != 0:
            chunk_to_write = min(8 - bit_i, bits_to_write)
            bitmask = (1 << chunk_to_write) - 1
            self.value[byte_i] |= (value & bitmask) << bit_i
            byte_i += 1
            bits_written = chunk_to_write

        for n in xrange(bits_written, bits_to_write, 8):
            chunk_to_write = min(8, bits_to_write - n)
            bitmask = (1 << chunk_to_write) - 1
            self.value[byte_i] |= (value >> n) & bitmask
            byte_i += 1
        
        self.position += bits_to_write

    def read(self, bits_to_read):
        if len(self.value) * 8 < (bits_to_read + self.position):
            raise IOError("attempting to read past the end of the buffer")
        
        byte_i, bit_i = (self.position / 8, self.position % 8)
        result = 0
        bits_read = 0

        if bit_i != 0:
            chunk_to_read = min(8 - bit_i, bits_to_read)
            bitmask = (1 << chunk_to_read) - 1
            result = (self.value[byte_i] >> bit_i) & bitmask
            bits_read = chunk_to_read
            byte_i += 1

        for n in xrange(bits_read, bits_to_read, 8):
            chunk_to_read = min(8, bits_to_read - n)
            bitmask = (1 << chunk_to_read) - 1
            result |= (self.value[byte_i] & bitmask) << n
            byte_i += 1

        self.position += bits_to_read
        return result

    def seek(self, position):
        self.position = position
        assert self.position >= 0 and ((self.position / 8) < len(self.value))



from functools import partial
from ..utils.kwargs import keep_kwargs_partial
from .range import SequentialRangeList, ByteRangeFactory

from .reference import (safe_repr, Reference, Context, ContextGetAttrReference, ObjectReference,
                        GetAttrReference, SetAndGetAttrReference, NumericSetAndGetAttrReference,
                        FuncCallReference, NumericFuncCallReference, NumericGetAttrReference, NumericReference)

from .buffer import (BufferType, FieldReference, NumericFieldReference, PackAbsolutePositionReference,
                     UnpackAbsolutePositionReference, TotalSizeReference)

from .serialize import (pack_int, unpack_int, pack_float, unpack_float, pack_str, unpack_str, pack_bytearray,
                        unpack_bytearray, pack_buffer, unpack_buffer, pack_list, unpack_list)

__all__ = [
    'int8', 'n_int8', 'b_int8', 'l_int8', 'uint8', 'n_uint8', 'b_uint8', 'l_uint8',
    'int16', 'n_int16', 'b_int16', 'l_int16', 'uint16', 'n_uint16', 'b_uint16', 'l_uint16',
    'int32', 'n_int32', 'b_int32', 'l_int32', 'uint32', 'n_uint32', 'b_uint32', 'l_uint32',
    'int64', 'n_int64', 'b_int64', 'l_int64', 'uint64', 'n_uint64', 'b_uint64', 'l_uint64',
    'int_field', 'float_field', 'str_type', 'str_type_factory', 'str_field', 'buffer_field', 'list_field',
    'bytearray_field', 'be_int_field', 'le_int_field', 'bytes_ref', 'total_size', 'after_ref', 'member_func_ref'
]
JUSTIFY = ('left', 'right')


def int_marshal(byte_size, sign, endian="native"):
    return (keep_kwargs_partial(pack_int, byte_size=byte_size, sign=sign, endian=endian),
            keep_kwargs_partial(unpack_int, byte_size=byte_size, sign=sign, endian=endian))

n_int8 = int8 = int_marshal(1, "signed", "native")
b_int8 = int_marshal(1, "signed", "big")
l_int8 = int_marshal(1, "signed", "little")

n_uint8 = uint8 = int_marshal(1, "unsigned", "native")
b_uint8 = int_marshal(1, "unsigned", "big")
l_uint8 = int_marshal(1, "unsigned", "little")

n_int16 = int16 = int_marshal(2, "signed", "native")
b_int16 = int_marshal(2, "signed", "big")
l_int16 = int_marshal(2, "signed", "little")

n_uint16 = uint16 = int_marshal(2, "unsigned", "native")
b_uint16 = int_marshal(2, "unsigned", "big")
l_uint16 = int_marshal(2, "unsigned", "little")

n_int32 = int32 = int_marshal(4, "signed", "native")
b_int32 = int_marshal(4, "signed", "big")
l_int32 = int_marshal(4, "signed", "little")

n_uint32 = uint32 = int_marshal(4, "unsigned", "native")
b_uint32 = int_marshal(4, "unsigned", "big")
l_uint32 = int_marshal(4, "unsigned", "little")

n_int64 = int64 = int_marshal(8, "signed", "native")
b_int64 = int_marshal(8, "signed", "big")
l_int64 = int_marshal(8, "signed", "little")

n_uint64 = uint64 = int_marshal(8, "unsigned", "native")
b_uint64 = int_marshal(8, "unsigned", "big")
l_uint64 = int_marshal(8, "unsigned", "little")

bytes_ref = ByteRangeFactory()
total_size = TotalSizeReference()


def str_type_factory(encoding='ascii', padding=' ', justify='left', byte_size=None):
    kwargs = dict(encoding=encoding, padding=padding, justify=justify)
    if byte_size:
        kwargs['byte_size'] = byte_size
    return (keep_kwargs_partial(pack_str, **kwargs), keep_kwargs_partial(unpack_str, **kwargs))

str_type = str_type_factory()


class InputBufferPartReference(Reference):
    def __init__(self, position_ref):
        self.position_ref = position_ref

    def evaluate(self, ctx):
        return ctx.input_buffer.get(self.position_ref(ctx))

    def __safe_repr__(self):
        return "input_buffer[{0!r}]".format(self.position_ref)


class UnpackerReference(FuncCallReference):
    def __init__(self, unpacker, absolute_position_ref, *args, **kwargs):
        super(UnpackerReference, self).__init__(unpacker, InputBufferPartReference(absolute_position_ref),
                                                *args, **kwargs)

    def _args_repr(self):
        return ", ".join(["buffer", "position"] + [safe_repr(arg) for arg in self.arg_refs[2:]])


class FieldUnpackerReference(Reference):
    def __init__(self, unpacker_ref):
        self.unpacker_ref = unpacker_ref

    def evaluate(self, ctx):
        return self.unpacker_ref.evaluate(ctx)[0]

    def __safe_repr__(self):
        return "field_unpack({0})".format(self.unpacker_ref._func_repr())


class NumericFieldUnpackerReference(FieldUnpackerReference, NumericReference):
    pass


class SequentialRangeListByteLengthReference(NumericFuncCallReference):
    def __init__(self, arg):
        super(SequentialRangeListByteLengthReference, self).__init__(SequentialRangeList.byte_length, arg)

    def __safe_repr__(self):
        return "byte_length_ref({0})".format(self._args_repr())


class PackSequentialRangeListByteLengthReference(SequentialRangeListByteLengthReference):
    """
    A short-circuit to make sure we don't end up in a cyclic reference if the packing position depends on the packing
    result.
    """
    def evaluate(self, ctx):
        if self.arg_refs[0].is_open(ctx):
            return None
        else:
            return super(PackSequentialRangeListByteLengthReference, self).evaluate(ctx)

    def __safe_repr__(self):
        return "pack_byte_length_ref({0})".format(self._args_repr())


class FieldReferenceBuilder(object):
    def __init__(self, numeric, set_before_pack, set_after_unpack, where, where_when_pack, where_when_unpack,
                 unpack_after, default):
        self.numeric = numeric
        self.set_before_pack = set_before_pack
        self.set_after_unpack = set_after_unpack
        self.where = where
        self.where_when_pack = where_when_pack
        self.where_when_unpack = where_when_unpack
        self.unpack_after = unpack_after
        self.default = default
        self.pack_size = None
        self.unpack_size = None

        self.get_obj_from_ctx_ref = ContextGetAttrReference('obj')
        self.set_and_get_class = NumericSetAndGetAttrReference if self.numeric else SetAndGetAttrReference

        self.resolve_static_where()
        self.create_field()
        self.set_field_position()
        self.set_field_pack_value_ref()

    def resolve_static_where(self):
        def resolve_static_ref(ref):
            range_list = None
            if ref.is_static():
                range_list = ref(Context())
            return range_list.byte_length() if range_list is not None else None

        if self.where is not None:
            assert self.where_when_pack is None and self.where_when_unpack is None
            self.pack_size = self.unpack_size = resolve_static_ref(self.where)

        if self.where_when_pack is not None:
            assert self.where is None and self.where_when_unpack is not None
            self.pack_size = resolve_static_ref(self.where_when_unpack)

        if self.where_when_unpack is not None:
            assert self.where is None and self.where_when_pack is not None
            self.unpack_size = resolve_static_ref(self.where_when_unpack)

        if self.pack_size is not None and self.unpack_size is not None:
            assert self.pack_size == self.unpack_size

    def create_field(self):
        field_class = NumericFieldReference if self.numeric else FieldReference
        self.field = field_class()
        # When we first create a field reference we don't know the field name yet. When __new__ will get called
        # on Buffer, it will fill it in for us.
        self.field.attr_name_ref = ObjectReference(None)
        self.field.default = self.default

    def set_field_position(self):
        if self.where is not None:
            pack_position_ref = unpack_position_ref = self.where
        else:
            pack_position_ref = self.where_when_pack
            unpack_position_ref = self.where_when_unpack

        self.field.pack_absolute_position_ref = PackAbsolutePositionReference(self.field, pack_position_ref)
        self.field.unpack_absolute_position_ref = UnpackAbsolutePositionReference(self.field, unpack_position_ref)

    def set_packer(self, packer, **kwargs):
        pack_kwargs = dict(byte_size=PackSequentialRangeListByteLengthReference(self.field.pack_absolute_position_ref),
                           pack_size=self.pack_size)
        pack_kwargs.update(kwargs)
        self.field.pack_ref = FuncCallReference(packer, self.field.pack_value_ref, **pack_kwargs)

    def set_unpacker(self, unpacker, **kwargs):
        unpack_kwargs = dict(byte_size=SequentialRangeListByteLengthReference(self.field.unpack_absolute_position_ref),
                             unpack_size=self.unpack_size)
        unpack_kwargs.update(kwargs)

        self.field.unpack_ref = UnpackerReference(unpacker, self.field.unpack_absolute_position_ref, **unpack_kwargs)

    def create(self):
        self.set_field_unpack_value_ref()
        self.set_field_unpack_after()
        return self.field

    def set_field_pack_value_ref(self):
        if self.set_before_pack is not None:
            if not isinstance(self.set_before_pack, Reference):
                pack_value_class = NumericFuncCallReference if self.numeric else FuncCallReference
                pack_value_ref = pack_value_class(self.set_before_pack, self.get_obj_from_ctx_ref)
            else:
                pack_value_ref = self.set_before_pack
            self.field.pack_value_ref = self.set_and_get_class(self.get_obj_from_ctx_ref, self.field.attr_name_ref, pack_value_ref)
        else:
            getter_ref_class = NumericGetAttrReference if self.numeric else GetAttrReference
            self.field.pack_value_ref = getter_ref_class(self.get_obj_from_ctx_ref, self.field.attr_name_ref)

    def set_field_unpack_value_ref(self):
        if self.numeric:
            field_unpack_ref = NumericFieldUnpackerReference(self.field.unpack_ref)
        else:
            field_unpack_ref = FieldUnpackerReference(self.field.unpack_ref)
        if self.set_after_unpack is not None:
            if not isinstance(self.set_after_unpack, Reference):
                unpack_value_class = NumericFuncCallReference if self.numeric else FuncCallReference
                unpack_value_ref = unpack_value_class(self.set_after_unpack, field_unpack_ref)
        else:
            unpack_value_ref = self.set_and_get_class(self.get_obj_from_ctx_ref, self.field.attr_name_ref, field_unpack_ref)

        self.field.unpack_value_ref = unpack_value_ref

    def set_field_unpack_after(self):
        if self.unpack_after is None:
            self.unpack_after = []
        elif not isinstance(self.unpack_after, (list, tuple)):
            self.unpack_after = [self.unpack_after]

        for unpack_after_field in self.unpack_after:
            assert isinstance(unpack_after_field, FieldReference)

        self.field.unpack_after = self.unpack_after


def int_field(endian='native', sign='signed',
              set_before_pack=None,
              set_after_unpack=None,
              where=None,
              where_when_pack=None,
              where_when_unpack=None,
              unpack_after=None,
              byte_size=None,
              default=None):
    builder = FieldReferenceBuilder(numeric=True,
                                    set_before_pack=set_before_pack,
                                    set_after_unpack=set_after_unpack,
                                    where=where,
                                    where_when_pack=where_when_pack,
                                    where_when_unpack=where_when_unpack,
                                    unpack_after=unpack_after,
                                    default=default)

    marshal_kwargs = dict(sign=sign, endian=endian)
    if byte_size:
        marshal_kwargs['byte_size'] = byte_size
    builder.set_packer(pack_int, **marshal_kwargs)
    builder.set_unpacker(unpack_int, **marshal_kwargs)
    return builder.create()


def be_int_field(*args, **kwargs):
    kwargs.update(dict(endian='big'))
    return int_field(*args, **kwargs)


def le_int_field(*args, **kwargs):
    kwargs.update(dict(endian='little'))
    return int_field(*args, **kwargs)


def float_field(endian='native',
                set_before_pack=None,
                set_after_unpack=None,
                where=None,
                where_when_pack=None,
                where_when_unpack=None,
                unpack_after=None,
                byte_size=None,
                default=None):
    builder = FieldReferenceBuilder(numeric=True,
                                    set_before_pack=set_before_pack,
                                    set_after_unpack=set_after_unpack,
                                    where=where,
                                    where_when_pack=where_when_pack,
                                    where_when_unpack=where_when_unpack,
                                    unpack_after=unpack_after,
                                    default=default)
    marshal_kwargs = dict(endian=endian)
    if byte_size:
        marshal_kwargs['byte_size'] = byte_size
    builder.set_packer(pack_float, **marshal_kwargs)
    builder.set_unpacker(unpack_float, **marshal_kwargs)
    return builder.create()


def bytearray_field(set_before_pack=None,
                    set_after_unpack=None,
                    where=None,
                    where_when_pack=None,
                    where_when_unpack=None,
                    unpack_after=None,
                    default=None):
    builder = FieldReferenceBuilder(numeric=False,
                                    set_before_pack=set_before_pack,
                                    set_after_unpack=set_after_unpack,
                                    where=where,
                                    where_when_pack=where_when_pack,
                                    where_when_unpack=where_when_unpack,
                                    unpack_after=unpack_after,
                                    default=default)
    builder.set_packer(pack_bytearray)
    builder.set_unpacker(unpack_bytearray)
    return builder.create()


def str_field(encoding='ascii', padding=' ', justify='left',
              set_before_pack=None,
              set_after_unpack=None,
              where=None,
              where_when_pack=None,
              where_when_unpack=None,
              unpack_after=None,
              default=None):
    builder = FieldReferenceBuilder(numeric=False,
                                    set_before_pack=set_before_pack,
                                    set_after_unpack=set_after_unpack,
                                    where=where,
                                    where_when_pack=where_when_pack,
                                    where_when_unpack=where_when_unpack,
                                    unpack_after=unpack_after,
                                    default=default)
    marshal_kwargs = dict(encoding=encoding, padding=padding, justify=justify)
    builder.set_packer(pack_str, **marshal_kwargs)
    builder.set_unpacker(unpack_str, **marshal_kwargs)
    return builder.create()


def unpack_selector_decorator(selector):
    def my_selector(obj, buffer, *args, **kwargs):
        o = selector(obj, buffer, *args, **kwargs)
        if isinstance(o, BufferType):
            result = o()
            byte_size = result.unpack(buffer)
            return result, byte_size
        elif isinstance(o, (list, tuple)):
            assert len(o) == 2, "selector returned a list but not a (obj, byte_size)-kind of list"
            return o
        elif o is None:
            return None, 0
        else:
            assert False, "selector didn't return a Buffer type, a pair of (obj, byte_size) or None, instead it returned {0!r}".format(o)
    return FuncCallReference(partial, my_selector, ContextGetAttrReference('obj'))


def buffer_field(type,
                 unpack_selector=None,
                 set_before_pack=None,
                 set_after_unpack=None,
                 where=None,
                 where_when_pack=None,
                 where_when_unpack=None,
                 unpack_after=None,
                 byte_size=None,
                 default=None):
    builder = FieldReferenceBuilder(numeric=False,
                                    set_before_pack=set_before_pack,
                                    set_after_unpack=set_after_unpack,
                                    where=where,
                                    where_when_pack=where_when_pack,
                                    where_when_unpack=where_when_unpack,
                                    unpack_after=unpack_after,
                                    default=default)
    marshal_kwargs = dict(type=type)
    if byte_size:
        marshal_kwargs = dict(byte_size=byte_size)
    builder.set_packer(pack_buffer, **marshal_kwargs)
    unpack = unpack_selector_decorator(unpack_selector) if unpack_selector else unpack_buffer
    builder.set_unpacker(unpack, **marshal_kwargs)
    return builder.create()


def list_field(type, n=None, unpack_selector=None,
               set_before_pack=None,
               set_after_unpack=None,
               where=None,
               where_when_pack=None,
               where_when_unpack=None,
               unpack_after=None,
               default=None):
    assert isinstance(type, (tuple, BufferType)), \
        "list type argument must be one of the predefined types or a subclass of Buffer but instead it's {0}".format(type)

    builder = FieldReferenceBuilder(numeric=False,
                                    set_before_pack=set_before_pack,
                                    set_after_unpack=set_after_unpack,
                                    where=where,
                                    where_when_pack=where_when_pack,
                                    where_when_unpack=where_when_unpack,
                                    unpack_after=unpack_after,
                                    default=default)

    shared_kwargs = dict(n=n)

    if isinstance(type, tuple):
        elem_packer, elem_unpacker = type
    else:
        elem_packer, elem_unpacker = (pack_buffer, partial(unpack_buffer, type=type))

    builder.set_packer(pack_list, elem_packer=elem_packer, **shared_kwargs)
    if unpack_selector:
        builder.set_unpacker(unpack_list, elem_unpacker=unpack_selector_decorator(unpack_selector), **shared_kwargs)
    else:
        builder.set_unpacker(unpack_list, elem_unpacker=elem_unpacker, **shared_kwargs)
    return builder.create()


class AfterFieldReference(Reference, NumericReference):
    def __init__(self, field_ref):
        self.field_ref = field_ref

    def evaluate(self, ctx):
        offset = None
        if ctx.is_pack():
            offset = SequentialRangeList(self.field_ref.pack_absolute_position_ref(ctx)).max_stop()
        else:
            # This is a bit more tricky, since unpack_absolute_position_ref returns the length for the rest of the
            # buffer if it contains an open range.
            if self.field_ref.unpack_absolute_position_ref.is_open(ctx):
                # It's an open range, so we need to limit it by checking the _actual_ byte size the field used to unpack.
                # Since all reference results are cached, we can ask the unpack result again w/o performance problems.
                _, byte_size = self.field_ref.unpack_ref(ctx)
                position_list = self.field_ref.unpack_absolute_position_ref.unpack_position_ref(ctx)
                # Since we want the "highest" byte, we need to sort the ranges.
                offset = position_list.sorted().byte_offset(byte_size)
            else:
                offset = self.field_ref.unpack_absolute_position_ref(ctx).max_stop()
        return offset

    def __safe_repr__(self):
        return "after({0})".format(self.field_ref.attr_name_ref(Context()))


def after_ref(field_ref):
    return AfterFieldReference(field_ref)


def member_func_ref(func):
    return FuncCallReference(func, ContextGetAttrReference('obj'))



class ReadAheadStream(object):
    def __init__(self, stream):
        self.buffer = ''
        self.stream = stream
        self.read_ahead_mode = False

    def set_read_ahead(self, mode):
        self.read_ahead_mode = mode

    def is_read_ahead_empty(self):
        return len(self.buffer) == 0
        
    def read(self, size):
        assert size >= 0

        res = ''
        if self.read_ahead_mode:
            res += self.stream.read(size)
            self.buffer += res
        else:
            if len(self.buffer) > 0:
                buffer_bytes_to_read = min(len(self.buffer), size)
                res = self.buffer[0:buffer_bytes_to_read]
                self.buffer = self.buffer[buffer_bytes_to_read:]
            if len(res) < size:
                res += self.stream.read(size - len(res))
        return res
    
    def write(self, buf):
        if len(self.buffer) > 0:
            raise IOError("cannot write while there are still bytes in the read-ahead buffer")
        if self.read_ahead:
            raise IOError("cannot write while in read-ahead mode")
        return self.stream.write(buf)



from .array import *
from .struct import Field

def FixedSizeArray(name, n, element_io, default=None):
    if default is not None:
        assert len(default) == n
    return Field(name, FixedSizeArrayMarshal(n, element_io), default)

def SumSizeArray(name, size_io, element_io, default=None):
    return Field(name, SumSizeArrayMarshal(size_io, element_io), default)

def VarSizeArray(name, size_io, element_io, default=None):
    return Field(name, VarSizeArrayMarshal(size_io, element_io), default)




from cStringIO import StringIO

from . import FieldBase, Field, FieldListContainer
from ..base import FixedSizer, EMPTY_CONTEXT

class LazyField(Field):
    def __init__(self, container, field):
        super(LazyField, self).__init__(field.name, field.default, field.marshal)
        self.container = container
        self.field = field

    def prepare_class(self, cls):
        setattr(cls, self.field.name, self)

    def prepare_instance(self, obj, args, kwargs):
        if self.name in kwargs:
            setattr(obj, self.name, kwargs.pop(self.name))
        
    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        self.field.write_fields(obj, stream, context)

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        self.field.read_fields(obj, stream, context, *args, **kwargs)
        
    def __set__(self, instance, value):
        self.container.instantiate_if_needed(instance)
        instance.__dict__[self.name] = value
    
    def __get__(self, instance, owner):
        self.container.instantiate_if_needed(instance)
        return instance.__dict__[self.name]

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        if self.container.is_instantiated(obj):
            return self.field.to_repr(obj, context)
        return "<lazy>"

    def sizeof(self, obj):
        return self.field.sizeof(obj)

    def min_max_sizeof(self):
        return self.field.min_max_sizeof()

class LazyFieldListContainer(FixedSizer, FieldListContainer):
    def __init__(self, fields):
        new_fields = []
        for field in fields:
            if not field.is_fixed_size():
                raise ValueError("%s must be a fixed-size field inside a lazy container" % (field,))
            if isinstance(field, Field):
                new_fields.append(LazyField(self, field))
            else:
                new_fields.append(field)
        
        super(LazyFieldListContainer, self).__init__(new_fields)

        self.size = sum([ field.min_max_sizeof().min for field in self.fields ])
        self.lazy_key = "_lazy_container_%s" % id(self)

    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        self.instantiate_if_needed(obj)
        super(LazyFieldListContainer, self).write_fields(obj, stream)

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        data = stream.read(self.size)
        setattr(obj, self.lazy_key, dict(data=data, context=context, args=args, kwargs=kwargs))

    def is_instantiated(self, obj):
        return not hasattr(obj, self.lazy_key)

    def instantiate_if_needed(self, obj):
        if self.is_instantiated(obj):
            return

        params = getattr(obj, self.lazy_key)
        stream = StringIO(params['data'])

        # We first delete the lazy key so it'll return "instantiated" when the field descriptors are called.
        delattr(obj, self.lazy_key)
        
        super(LazyFieldListContainer, self).read_fields(obj, stream, params['context'], *params['args'],
                                                        **params['kwargs'])



from ..base import MinMax, EMPTY_CONTEXT
from . import Field

class OptionalField(Field):
    def __init__(self, name, marshal, predicate, default=None):
        super(OptionalField, self).__init__(name, marshal, default)
        self.predicate = predicate

    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        value = self.get_value(obj)
        if value is not None:
            self.marshal.write_to_stream(value, stream, context)

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        if self.predicate(obj, stream, context):
            super(OptionalField, self).read_fields(obj, stream, context, *args, **kwargs)

    def sizeof(self, obj):
        value = self.get_value(obj)
        if value is None:
            return 0
        return self.marshal.sizeof(value)
        
    def min_max_sizeof(self):
        return MinMax(0, self.marshal.min_max_sizeof().max)



from numbers import Number
import operator

OPERATOR_TO_SYMBOL = {
    operator.add: "+",
    operator.sub: "-",
    operator.neg: "-"
}


def safe_repr(obj):
    try:
        obj_repr = repr(obj)
    except:
        obj_repr = "({0}<{1}> repr error)".format(type(obj), id(obj))
    return obj_repr


def _repr_peel_obj_ref_for_str(ref):
    # Used to print attr names or other strings that shouldn't have the all ref('...') wrapping:
    if isinstance(ref, ObjectReference) and isinstance(ref.obj, str):
        return ref.obj
    else:
        return repr(ref)


class CyclicReferenceError(Exception):
    """Raised when discovering a cyclic reference."""

    MESSAGE = """Cyclic reference detected.
Reference cycle:
{cycle_str}
"""

    def __init__(self, ctx, last_ref):
        # Format the cycle as cycle_ref_1 -->\n cycle_ref_2 -->\n cycle_ref_3 -->\n...
        cycle = ["  {0!r}".format(ref) for ref in ctx.call_stack + [last_ref]]
        cycle_str = " -->\n".join(cycle)
        super(CyclicReferenceError, self).__init__(CyclicReferenceError.MESSAGE.format(cycle_str=cycle_str))


class Context(object):
    """Base class for working with references."""

    def __init__(self):
        self.cached_results = dict()
        self.call_nodes = set()
        self.call_stack = []
        self.exception_call_stack = None

    def format_exception_call_stack(self):
        return [repr(line) for line in self.exception_call_stack] if self.exception_call_stack is not None else []


class Reference(object):
    """
    An abstract reference. A reference is an object that will be resolved during packing or unpacking.
    This base class provides some convenience methods to handle references and expects subclasses to implement
    the value(obj) method.

    Be sure to override the following methods:
     * evaluate(self, ctx)
     * __safe_repr__(self)
    """

    def __call__(self, ctx):
        """
        Returns the value this reference is pointing to. This method uses 'obj' to resolve the reference and return
        the value this reference references.
        If the call was already made, it returns a cached result.
        It also makes sure there's no cyclic reference, and if so raises CyclicReferenceError.
        """
        if self in ctx.call_nodes:
            raise CyclicReferenceError(ctx, self)

        if self in ctx.cached_results:
            return ctx.cached_results[self]

        try:
            ctx.call_nodes.add(self)
            ctx.call_stack.append(self)

            result = self.evaluate(ctx)
            ctx.cached_results[self] = result
            return result
        except:
            if ctx.exception_call_stack is None:
                ctx.exception_call_stack = list(ctx.call_stack)
            raise
        finally:
            ctx.call_stack.pop()
            ctx.call_nodes.remove(self)

    def evaluate(self, ctx):
        raise NotImplementedError("Reference is an abstract class, you need to override evaluate(self, ctx) in {0}".
                                  format(type(self)))

    def __repr__(self):
        try:
            return self.__safe_repr__()
        except:
            return "({0}<{1}> repr error)".format(type(self), id(self))

    @classmethod
    def to_ref(self, ref):
        if isinstance(ref, Reference):
            return ref
        if isinstance(ref, Number):
            return NumberReference(ref)
        return ObjectReference(ref)

    def is_static(self):
        # We want to see if this reference can resolve w/o external dependencies, i.e. doesn't need anything from
        # the context. To do that, we have a "fake" context that raises an exception if someone tries to fetch
        # an attribute from it.
        class NotStaticObjectError(Exception):
            pass

        class StaticContext(Context):
            def __getattr__(self, name):
                raise NotStaticObjectError()

        ctx = StaticContext()
        try:
            self(ctx)
            return True
        except NotStaticObjectError:
            return False


class NumericReference(object):
    """
    A numeric reference is a mixin that adds support for arithmetic operators, such as add/sub.
    It supports operations on itself (NumericReference) and operations involving numbers and numeric references.
    """

    def __add__(self, other):
        if not isinstance(other, (Number, NumericReference)):
            return NotImplemented
        return NumericBinaryExpression(self, other, operator.add)

    def __radd__(self, other):
        if not isinstance(other, (Number, NumericReference)):
            return NotImplemented
        return NumericBinaryExpression(other, self, operator.add)

    def __sub__(self, other):
        if not isinstance(other, (Number, NumericReference)):
            return NotImplemented
        return NumericBinaryExpression(self, other, operator.sub)

    def __rsub__(self, other):
        if not isinstance(other, (Number, NumericReference)):
            return NotImplemented
        return NumericBinaryExpression(other, self, operator.sub)

    def __neg__(self):
        return NumericUnaryExpression(self, operator.neg)

    # FIXME: add rest of the operators, including unary ones


class NumericUnaryExpression(Reference, NumericReference):
    """
    Unary expression support for references, e.g. -ref('field').
    """
    def __init__(self, ref, operator):
        self.ref = Reference.to_ref(ref)
        self.operator = operator

    def evaluate(self, ctx):
        return self.operator(self.ref(ctx))

    def __safe_repr__(self):
        op_sym = OPERATOR_TO_SYMBOL[self.operator] if self.operator in OPERATOR_TO_SYMBOL else repr(self.operator)
        return "{0}({1!r})".format(op_sym, self.ref)


class NumericBinaryExpression(Reference, NumericReference):
    """
    Binary expression support for references, e.g. ref('field') + 6.
    """
    def __init__(self, a, b, operator):
        self.a = Reference.to_ref(a)
        self.b = Reference.to_ref(b)
        self.operator = operator

    def evaluate(self, ctx):
        return self.operator(self.a(ctx), self.b(ctx))

    def __safe_repr__(self):
        op_sym = OPERATOR_TO_SYMBOL[self.operator] if self.operator in OPERATOR_TO_SYMBOL else repr(self.operator)
        return "({0!r} {1} {2!r})".format(self.a, op_sym, self.b)


class ObjectReference(Reference):
    """Holds a reference to an object."""

    def __init__(self, obj):
        self.obj = obj

    def evaluate(self, ctx):
        return self.obj

    def __safe_repr__(self):
        return "ref({0!r})".format(self.obj)


class NumberReference(ObjectReference, NumericReference):
    """Holds a reference to an object that we know is also a number."""

    def __safe_repr__(self):
        return "num_ref({0!r})".format(self.obj)


class FuncCallReference(Reference):
    """Holds a reference to a function call that will get executed when resolving."""

    def __init__(self, func, *args, **kwargs):
        self.func_ref = Reference.to_ref(func)
        self.arg_refs = [Reference.to_ref(arg) for arg in args]
        self.kwarg_refs = dict((k, Reference.to_ref(v)) for k, v in kwargs.items())

    def evaluate(self, ctx):
        func = self.func_ref(ctx)
        assert callable(func), "func {0!r} from func_ref {1!r} is not callable".format(func, self.func_ref)
        args = [arg_ref(ctx) for arg_ref in self.arg_refs]
        kwargs = dict((k, v_ref(ctx)) for k, v_ref in self.kwarg_refs.items())
        return func(*args, **kwargs)

    def __safe_repr__(self):
        return "func_ref({0})".format(self._func_repr())

    def _func_repr(self):
        # Small hacks to make repr look better:
        if isinstance(self.func_ref, ObjectReference):
            func_ref_repr = self.func_ref.obj.func_name
        else:
            func_ref_repr = safe_repr(self.func_ref)
        return "{0}({1}))".format(func_ref_repr, self._args_and_kwargs_repr())

    def _args_and_kwargs_repr(self):
        return ", ".join((self._args_repr(), self._kwargs_repr()))

    def _args_repr(self):
        return ", ".join(safe_repr(arg) for arg in self.arg_refs)

    def _kwargs_repr(self):
        return ", ".join("{0}={1}".format(k, safe_repr(v_ref)) for k, v_ref in self.kwarg_refs.items())


class NumericFuncCallReference(FuncCallReference, NumericReference):
    """Holds a reference to a function call that returns a number."""

    def __safe_repr__(self):
        return "num_func_ref({0})".format(self._func_repr())


class LengthFuncCallReference(NumericFuncCallReference):
    """Holds a reference to len(x) function call."""

    def __init__(self, arg):
        super(LengthFuncCallReference, self).__init__(len, arg)

    def __safe_repr__(self):
        return "len_ref({0!r})".format(self.arg_refs[0])


class GetAttrReference(FuncCallReference):
    """Holds a reference to getattr(object, attr_name) function call."""

    def __init__(self, object, attr_name):
        super(GetAttrReference, self).__init__(getattr, object, attr_name)

    def __safe_repr__(self):
        # small hacks to make repr look better:
        arg_ref_repr = _repr_peel_obj_ref_for_str(self.arg_refs[1])
        return "{0!r}.{1}".format(self.arg_refs[0], arg_ref_repr)


class NumericGetAttrReference(GetAttrReference, NumericReference):
    """Holds a reference to getattr(object, attr_name) function call where the attribute is a number."""
    pass


class SetAttrReference(FuncCallReference):
    """Holds a reference to setattr(object, attr_name, value) function call."""

    def __init__(self, object, attr_name, value):
        super(SetAttrReference, self).__init__(setattr, object, attr_name, value)

    def __safe_repr__(self):
        # small hacks to make repr look better:
        arg_ref_repr = _repr_peel_obj_ref_for_str(self.arg_refs[1])
        return "{0!r}.{1} = {2!r}".format(self.arg_refs[0], arg_ref_repr, self.arg_refs[2])


class SetAndGetAttrReference(FuncCallReference):
    """Holds a reference to setattr(object, attr_name, value) function call that returns value."""

    def __init__(self, object, attr_name, value):
        super(SetAndGetAttrReference, self).__init__(lambda obj, attr, val: setattr(obj, attr, val) or val,
                                                     object, attr_name, value)

    def __safe_repr__(self):
        arg_ref_repr = _repr_peel_obj_ref_for_str(self.arg_refs[1])
        return "set {0!r}.{1} = {2!r} then return rvalue".format(self.arg_refs[0], arg_ref_repr, self.arg_refs[2])


class NumericSetAndGetAttrReference(SetAndGetAttrReference, NumericReference):
    """Holds a reference to setattr(object, attr_name, value) function call that returns a numeric value."""
    pass


class ReturnContextReference(Reference):
    """A reference that evaluates to the context itself (useful when using FuncCallReference)."""

    def evaluate(self, ctx):
        return ctx

    def __safe_repr__(self):
        return "ctx"


class ContextGetAttrReference(GetAttrReference):
    """A reference that returns an attribute from the context."""

    def __init__(self, attr_name):
        super(ContextGetAttrReference, self).__init__(ReturnContextReference(), attr_name)



import math
import collections


def float_to_byte_bit_pair(n):
    return (int(n), int((n - int(n)) * 8))


class BitView(collections.Sequence):
    """
    Provides bit indexing over an existing buffer.

    For example, you can do this:
    bit_view = BitView(...)
    bits_2_and_3 = bit_view[0.125 * 2:0.125 * 4]

    Basically, you can access the buffer with 1/8 fractions.
    """

    def __init__(self, buffer, start=0, stop=None):
        super(BitView, self).__init__()
        # FIXME: On Python 2.7 there's no bytes() type (immutable byte sequence).
        if isinstance(buffer, bytearray):
            self.buffer = buffer
        elif isinstance(buffer, (str, unicode)):
            self.buffer = [ord(c) for c in buffer]
        else:
            self.buffer = buffer
        self.start = start if start is not None else 0
        self.stop = stop if stop is not None else len(buffer)
        assert self.start >= 0
        assert self.stop <= len(buffer)
        assert self.start <= self.stop

    def __getitem__(self, key):
        start, stop = self._key_to_range(key)
        if isinstance(key, slice):
            return self._get_range(start, stop)
        else:  # must be int/float otherwise _key_to_range would raise an error
            return self._get_byte_bits(start, min(8, int((self.stop - start) * 8)))

    def __len__(self):
        return int(math.ceil(self.stop - self.start))

    def __iter__(self):
        for i in xrange(len(self)):
            yield self._get_byte_bits(i + self.start, min(int((self.stop - i - self.start) * 8), 8))

    def __str__(self):
        return str(bytearray(b for b in self))

    def __repr__(self):
        return "{0}(buffer={1!r}, start={2}, stop={3})".format(type(self), self.buffer, self.start, self.stop)

    def to_bitstr(self):
        result = []
        for i in xrange(int(self.start * 8), int(self.stop * 8)):
            byte_i, bit_i = float_to_byte_bit_pair(float(i) / 8)
            result.append((self.buffer[byte_i] >> bit_i) & 1)
        return "".join(str(n) for n in reversed(result))

    def length(self):
        return self.stop - self.start

    def _get_range(self, start, stop):
        start_byte_ofs = int(start)
        stop_byte_ofs = int(math.ceil(stop))
        return type(self)(self.buffer[start_byte_ofs:stop_byte_ofs], start - start_byte_ofs, stop - start_byte_ofs)

    def _get_byte_bits(self, ofs, bit_len):
        # assert ofs >= 0 and bit_len >= 0, "ofs={0!r}, bit_len={1!r}".format(ofs, bit_len)
        byte_ofs, bit_ofs = float_to_byte_bit_pair(ofs)
        bit_mask = ((1 << bit_len) - 1)
        if bit_ofs == 0:
            return self.buffer[byte_ofs] & bit_mask
        elif bit_ofs < 0:
            bit_mask = ((1 << (bit_len + bit_ofs)) - 1)
            return self.buffer[0] & bit_mask
        elif bit_ofs + bit_len <= 8:
            return (self.buffer[byte_ofs] >> bit_ofs) & bit_mask
        else:
            cur_byte = self.buffer[byte_ofs]
            next_byte = self.buffer[byte_ofs + 1] if byte_ofs + 1 < len(self.buffer) else 0
            return (((cur_byte >> bit_ofs) & 0xFF) | ((next_byte << (8 - bit_ofs)) & 0xFF)) & bit_mask

    def _key_to_range(self, key):
        if isinstance(key, slice):
            if key.step not in (None, 1):
                raise NotImplementedError("step must be 1 or None")
            start = self._translate_offset(key.start if key.start is not None else 0)
            stop = self._translate_offset(key.stop if key.stop is not None else self.length())
            assert start <= stop and start >= 0, "start={0!r}, stop={1!r}".format(start, stop)
        elif isinstance(key, (float, int)):
            start = self._translate_offset(key)
            stop = start + 1
        else:
            raise TypeError("index must be int, float or a slice")
        return start, stop

    def _translate_offset(self, ofs):
        length = self.length()
        ofs = max(ofs + length, 0) if ofs < 0 else min(ofs, length)
        return ofs + self.start


class BitAwareByteArray(BitView, collections.MutableSequence):
    """
    Similar to BitView, but this class is mutable.
    """
    def __init__(self, source, start=0, stop=None):
        assert isinstance(source, bytearray)
        super(BitAwareByteArray, self).__init__(source, start, stop)

    def __setitem__(self, key, value):
        start, stop = self._key_to_range(key)
        value, value_len = self._value_to_value_and_length(value, stop - start)
        self._set_range(start, stop, value, value_len)

    def __delitem__(self, key):
        start, stop = self._key_to_range(key)
        self._del_range(start, stop)

    def insert(self, i, value):
        i = self._translate_offset(i)
        value, value_len = self._value_to_value_and_length(value)
        self._insert_zeros(i, i + value_len)
        self._copy_to_range(i, value, value_len)

    def extend(self, other):
        if isinstance(other, BitView):
            offset = self.stop
            self._insert_zeros(offset, offset + other.length())
            self._copy_to_range(offset, other, other.length())
        else:
            super(BitAwareByteArray, self).extend(other)

    def zfill(self, length):
        if length > self.length():
            self._insert_zeros(self.stop, self.stop + length - self.length())

    def __add__(self, other):
        if not isinstance(other, BitView):
            return NotImplemented
        copy = BitAwareByteArray(bytearray(self.buffer), start=self.start, stop=self.stop)
        copy.extend(other)
        return copy

    def __radd__(self, other):
        if not isinstance(other, BitView):
            return NotImplemented

        copy = BitAwareByteArray(bytearray(other.buffer), start=other.start, stop=other.stop)
        copy.extend(self)
        return copy

    def _set_range(self, start, stop, value, value_len):
        """
        Assumes that start and stop are already in 'buffer' coordinates. value is a byte iterable.
        value_len is fractional.
        """
        assert stop >= start and value_len >= 0
        range_len = stop - start
        if range_len < value_len:
            self._insert_zeros(stop, stop + value_len - range_len)
            self._copy_to_range(start, value, value_len)
        elif range_len > value_len:
            self._del_range(stop - (range_len - value_len), stop)
            self._copy_to_range(start, value, value_len)
        else:
            self._copy_to_range(start, value, value_len)

    def _copy_to_range(self, offset, iterable, iterable_len):
        remaining_bit_len = int(iterable_len * 8)
        for byte in iterable:
            self._set_byte_bits(offset, min(remaining_bit_len, 8), byte)
            offset += 1
            remaining_bit_len -= 8

    def _del_range(self, start, stop):
        assert stop >= start
        start_byte, stop_byte = int(math.ceil(start)), int(math.floor(stop))
        whole_byte_delta = stop_byte - start_byte

        # If we can remove whole bytes from the buffer, we'll do that first.
        if whole_byte_delta >= 1:
            del self.buffer[start_byte:stop_byte]
            self.stop -= whole_byte_delta
            stop -= whole_byte_delta

        # Here we have at most 8 bits to remove, so we need to "shift" the entire array.
        if stop > start:
            ofs = start
            bit_len_frac = stop - start
            while (ofs + bit_len_frac) < self.stop:
                self._set_byte_bits(ofs, 8, self._get_byte_bits(ofs + bit_len_frac, 8))
                ofs += 1
            self.stop -= bit_len_frac
            if int(math.ceil(self.stop)) < len(self.buffer):
                del self.buffer[-1]

    def _insert_zeros(self, start, stop):
        assert start >= 0 and start <= stop, "start={0!r}, stop={1!r}".format(start, stop)
        assert start <= self.stop
        start_byte, stop_byte = int(math.ceil(start)), int(math.floor(stop))
        whole_byte_delta = stop_byte - start_byte

        # If we can insert whole bytes to the buffer, we'll do that first.
        if whole_byte_delta >= 1:
            self.buffer[start_byte:start_byte] = bytearray(whole_byte_delta)
            self.stop += whole_byte_delta
            stop -= whole_byte_delta

        if stop > start:
            assert stop - start <= 2, "start={0}, stop={1}".format(start, stop)
            bit_len_frac = stop - start
            while int(math.ceil(self.stop + bit_len_frac)) > len(self.buffer):
                self.buffer.append(0)

            if start < self.stop:
                # Inserting in the middle, so we copy from end to start.
                ofs = self.stop + bit_len_frac - 1
                while ofs >= start:
                    self._set_byte_bits(ofs, 8, self._get_byte_bits(ofs - bit_len_frac, 8))
                    ofs -= 1
            self.stop += bit_len_frac

    def _set_byte_bits(self, ofs, bit_len, value):
        byte_ofs, bit_ofs = float_to_byte_bit_pair(ofs)
        if bit_ofs == 0 and bit_len == 8:
            self.buffer[byte_ofs] = value  # shortcut
        elif (bit_ofs + bit_len) <= 8:
            self.buffer[byte_ofs] &= ~(((1 << bit_len) - 1) << bit_ofs)
            self.buffer[byte_ofs] |= (value << bit_ofs) & 0xFF
        else:
            first_byte_bit_len = 8 - bit_ofs
            self._set_byte_bits(ofs, first_byte_bit_len, value & ((1 << first_byte_bit_len) - 1))
            self._set_byte_bits(byte_ofs + 1, bit_len - first_byte_bit_len, value >> first_byte_bit_len)

    def _value_to_value_and_length(self, value, int_value_len=1):
        value_len = 0
        if isinstance(value, BitView):
            value_len = value.length()
        elif isinstance(value, collections.Sized):
            value_len = len(value)
        elif isinstance(value, collections.Iterable):
            value = bytearray(value)
            value_len = len(value)
        elif isinstance(value, int):
            # Short circuit: make bit ranges accept int values by their bit length.
            bit_length = max(1, value.bit_length())
            if bit_length > int_value_len * 8:
                # Safety measure for short circuit: if user is assigning an int with more bits than the range of
                # bits that he specified we shout.
                raise ValueError("trying to assign int {0} with bit length {1} to bit length {2}".format(value,
                                 bit_length, int(int_value_len * 8)))
            l = []
            for n in range(0, bit_length, 8):
                l.append(value % 256)
                value /= 256
            value = l
            value_len = max(float(bit_length) / 8, int_value_len)
        else:
            raise TypeError("value must be iterable or int")
        return value, value_len


class InputBuffer(object):
    def __init__(self, buffer):
        self.buffer = BitView(buffer)

    def get(self, range_list):
        result = BitAwareByteArray(bytearray())
        for range in range_list:
            assert not range.is_open()
            result += self.buffer[range.start:range.stop]

        return result

    def length(self):
        return len(self.buffer)

    def __repr__(self):
        return "InputBuffer({0!r})".format(self.buffer)


class OutputBuffer(object):
    def __init__(self):
        self.buffer = BitAwareByteArray(bytearray())

    def set(self, value, range_list):
        value = BitView(value)
        value_start = 0

        for range in range_list:
            assert not range.is_open()
            assert range.start >= 0 and range.start <= range.stop
            if range.length() > len(value) - value_start:
                raise ValueError("trying to assign a value with smaller length than the range it's given")
            self.buffer.zfill(range.start)
            self.buffer[range.start:range.stop] = value[value_start:value_start + range.length()]
            value_start += range.length()

    def get(self):
        return self.buffer



from . import FieldListContainer
from ..base import FixedSizer, Marshal, MinMax, EMPTY_CONTEXT
from ..errors import InstructError, BitFieldNotInByteBoundry, FieldTypeNotSupportedError
from ..utils.bitstringio import BitStringIO

class BitMarshal(FixedSizer, Marshal):
    def __init__(self, size):
        super(BitMarshal, self).__init__()
        self.size = size

    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        return stream.read(self.size)

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        stream.write(obj, self.size)

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return context.get('int_repr_format', '%d') % obj

class BitPaddingMarshal(BitMarshal):
    def __init__(self, size):
        super(BitPaddingMarshal, self).__init__(size)

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        stream.write(0, self.size)

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return "<%d bits padding>" % (self.size,)

class BitFieldListContainer(FixedSizer, FieldListContainer):
    def __init__(self, fields):
        super(BitFieldListContainer, self).__init__(fields)
        if not all([ field.is_fixed_size() for field in self.fields ]):
            raise InstructError("all fields in a bit field list must be fixed size")
        self.size = sum([ field.min_max_sizeof().min for field in self.fields ])
        if (self.size % 8) != 0:
            raise BitFieldNotInByteBoundry()
        self.size /= 8
        
    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        bit_stream = BitStringIO(self.size)
        for io in self.fields:
            io.write_fields(obj, bit_stream, context)
        stream.write(bit_stream.getvalue())

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        bit_stream = BitStringIO(stream.read(self.size))
        for field in self.fields:
            field.read_fields(obj, bit_stream, context, *args, **kwargs)

class PositionalBitMarshal(FixedSizer, Marshal):
    def __init__(self, slices):
        super(PositionalBitMarshal, self).__init__()
        self.slices = slices
        self.size = sum([ s.stop - s.start for s in slices ])

    def get_slices(self):
        return self.slices

    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        value = 0
        bits = 0
        for s in self.slices:
            stream.seek(s.start)
            slice_size = s.stop - s.start
            value += stream.read(slice_size) << bits
            bits += slice_size
        return value

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        for s in self.slices:
            stream.seek(s.start)
            slice_size = s.stop - s.start
            stream.write(obj & ((1 << slice_size) - 1), slice_size)
            obj >>= slice_size

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return context.get('int_repr_format', '%d') % obj

class PositionalBitFieldListContainer(FixedSizer, FieldListContainer):
    def __init__(self, bit_size, fields):
        super(PositionalBitFieldListContainer, self).__init__(fields)
        if not all([ field.is_fixed_size() for field in self.fields ]):
            raise InstructError("all fields in a bit field list must be fixed size")

        field_bit_size = max([ max([ s.stop for s in field.marshal.get_slices() ]) for field in self.fields ])
        if bit_size is None:
            bit_size = field_bit_size

        if bit_size < field_bit_size:
            raise ValueError("bit size is set to %d but the fields index max bit %d" % (bit_size, field_bit_size))

        self.bit_size = bit_size
        if (self.bit_size % 8) != 0:
            raise BitFieldNotInByteBoundry()
        
        self.size = self.bit_size / 8
        
    def write_fields(self, obj, stream, context=EMPTY_CONTEXT):
        bit_stream = BitStringIO(self.size)
        for field in self.fields:
            field.write_fields(obj, bit_stream, context)
        stream.write(bit_stream.getvalue())

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        bit_stream = BitStringIO(stream.read(self.size))
        for field in self.fields:
            field.read_fields(obj, bit_stream, context, *args, **kwargs)



import functools

#from .struct import Struct, FieldAdapter
#from .struct.const import ConstFieldAdapter
#from .struct.bit import BitFieldListIO, BitIO, BitPaddingIO
#from .struct.lazy import LazyFieldListIO
#from .struct.optional import OptionalFieldAdapter
#from .struct.selector import StructSelectorIO, FuncStructSelectorIO
#from .array import SumSizeArrayIO, FixedSizeArrayIO
#from .padding import *
from .base import *
from .numeric_macros import *
from .string_macros import *
from .mapping_macros import *
from .array_macros import *
from .struct_macros import *

# def Field(name, io, default=None):
#     """
#     Macro helper to create a new `FieldAdapter` used in a `FieldListIO` instance.

#     :param name: name of the field
#     :param io: an I/O instance used to serializer/deserialize this field
#     :param default: default value of the field if not set by user
#     :rtype: FieldAdapter
#     """
#     return FieldAdapter(name, default, io)

# def ConstField(name, value, io=None):
#     """
#     Macro helper to create a new `ConstFieldAdapter` used in a `FieldListIO` instance.

#     This macro can be used in several methods:

#     >>> ConstField("foo", 5, UBInt8)

#     This created a constant field called ``foo`` with a value of 5 and is serialized/deserialized using UBInt8.

#     >>> ConstField("foo", MyStruct(my_field=1, my_other_field=2))

#     This time ``foo`` is set with the ``MyStruct`` instance passed here. Notice that we don't need to pass an I/O
#     argument because the value is an I/O instance by itself.

#     :param name: name of the field
#     :param value: the value to use as a constant
#     :param io: an I/O instance used to serializer/deserialize this field (optional if ``value`` is an I/O by itself)
#     :rtype: FieldAdapter
#     """
#     if io is None:
#         io = value
#     if isinstance(io, Struct):
#         io = io._io_
#     elif not (isinstance(io, Writer) and isinstance(io, AllocatingReader)):
#         raise InstructError("don't know how to serialize const field %s value %s (consider adding an io argument)" %
#                             (name, value))
#     return ConstFieldAdapter(name, value, io)

# def OptionalField(name, io, predicate, default=None):
#     return OptionalFieldAdapter(name, default, io, predicate)

# def BitField(name, size, default=None):
#     return FieldAdapter(name, default, BitIO(size))

# def BitPadding(size):
#     return BitPaddingIO(size)

# def BitFields(*args):
#     return BitFieldListIO(args)

# def BitFlag(name, default=None):
#     return BitField(name, 1, default)

# def SelfPredicate(func):
#     def wrapper(obj, stream, context):
#         return func(context.get('parent', None), obj, stream, context)
#     return wrapper

# # Backward compatibility
# Flag = BitFlag

# def BytePadding(size, char='\x00'):
#     return BytePaddingIO(size, char)

# # Backward compatibility
# Padding = BytePadding

# def Lazy(*args):
#     return LazyFieldListIO(args)

# def StructSelector(key_io, mapping, default=None):
#     return StructSelectorIO(key_io, mapping, default)

# def StructFunc(func):
#     def wrapper(stream, context):
#         return func(context.get('parent', None), stream, context)
#     return wrapper

# def SelectStructByFunc(name, func, min_max_size=None, default=None):
#     return FieldAdapter(name, default, FuncStructSelectorIO(StructFunc(func), min_max_size))




from .base import FixedSizer, Marshal, EMPTY_CONTEXT

class BytePaddingMarshal(FixedSizer, Marshal):
    def __init__(self, size, char="\x00"):
        super(BytePaddingMarshal, self).__init__()
        self.size = size
        self.char = char
        assert len(self.char) == 1

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        stream.write(self.char * self.size)

    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        stream.read(self.size)

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return "<%d bytes padding>" % (self.size,)



import types

from .struct import Struct, Field, AnonymousField
from .struct.optional import OptionalField
from .struct.const import ConstField as OrigConstField
from .struct.bit import BitFieldListContainer, BitMarshal, BitPaddingMarshal
from .struct.bit import PositionalBitMarshal, PositionalBitFieldListContainer
from .struct.lazy import LazyFieldListContainer
from .struct.selector import StructSelectorMarshal, FuncStructSelectorMarshal
from .struct.pointer import ReadPointer

from .base import Marshal, UNBOUNDED_MIN_MAX
from .errors import InstructError
from .padding import BytePaddingMarshal

def ConstField(name, value, marshal=None):
    """
    This macro can be used in several methods:

    >>> ConstField("foo", 5, UBInt8)

    This created a constant field called ``foo`` with a value of 5 and is serialized/deserialized using UBInt8.

    >>> ConstField("foo", MyStruct(my_field=1, my_other_field=2))

    This time ``foo`` is set with the ``MyStruct`` instance passed here. Notice that we don't need to pass an I/O
    argument because the value is an I/O instance by itself.

    :param name: name of the field
    :param value: the value to use as a constant
    :param marshal: a marshal instance to serialize/deserialize this field (optional if ``value`` is a marshal)
    :rtype: Field
    """
    if marshal is None:
        marshal = value
    if isinstance(marshal, Struct):
        marshal = type(marshal)
    elif not isinstance(marshal, Marshal):
        raise InstructError("don't know how to serialize const field %s value %s (consider adding a marshal argument)" %
                            (name, value))
    return OrigConstField(name, marshal, value)

def BitFields(*args):
    if any([ isinstance(field.marshal, PositionalBitMarshal) for field in args ]):
        assert all([ isinstance(field.marshal, PositionalBitMarshal) for field in args ])
        return PositionalBitFieldListContainer(None, args)
    else:
        return BitFieldListContainer(args)

def BitField(name, size, default=None):
    if isinstance(size, (types.TupleType, types.ListType, slice)):
        return Field(name, PositionalBitMarshal((size,) if isinstance(size, slice) else size), default)
    return Field(name, BitMarshal(size), default)

def BitFlag(name, default=None):
    return Field(name, BitMarshal(1), default)

def BitPadding(size):
    return AnonymousField(BitPaddingMarshal(size))

def BytePadding(size, char='\x00'):
    return AnonymousField(BytePaddingMarshal(size, char))

class BitSlicer(object):
    def __getitem__(self, key):
        if isinstance(key, (types.TupleType, types.ListType)):
            return list(key)
        elif isinstance(key, slice):
            return [ key ]

Bits = BitSlicer()

def Lazy(*args):
    return LazyFieldListContainer(args)

# Backward compatibility
Padding = BytePadding

def StructSelector(key_marshal, mapping, default=None):
    return StructSelectorMarshal(key_marshal, mapping, default)

def StructFunc(func):
    def wrapper(stream, context):
        return func(context.get('struct', None), stream, context)
    return wrapper

def SelectStructByFunc(name, func, min_max_size=UNBOUNDED_MIN_MAX, default=None):
    return Field(name, FuncStructSelectorMarshal(StructFunc(func), min_max_size), default)



from ..base import CallableReader

class ReadPointer(CallableReader):
    def __init__(self, name):
        super(ReadPointer, self).__init__(self._read_value)
        self.name = name

    def _read_value(self, stream, context):
        obj = context.get("struct", None)
        return getattr(obj, self.name)



import sys
import operator
import functools

from .reference import safe_repr, Reference, NumericReference

BIT = 0.125


@functools.total_ordering
class SequentialRange(object):
    """
    An almost slice-like object representing a sequential (continous) range from start to stop (exclusive).
    This class can only work on non-negative values, but allows stop to be None (open-ended).
    """
    def __init__(self, start, stop):
        self.start = start if start is not None else 0
        self.stop = stop

        assert self.start >= 0, "start={0!r}".format(self.start)
        if self.stop is not None:
            assert self.stop >= 0, "stop={0!r}".format(self.stop)
            assert self.start <= self.stop, "start={0!r}, stop={1!r}".format(self.start, self.stop)

    def is_open(self):
        """Returns True if the range is open, i.e. stop is None"""
        return self.stop is None

    def to_closed(self, total_len):
        """Converts a potentially open range into a closed range by making stop a finite value (total_len)."""
        assert total_len >= self.start, "self.start ({0}) is bigger than total_len ({1})".format(self.start, total_len)
        return SequentialRange(self.start, self.stop if self.stop is not None else total_len)

    def length(self):
        if self.is_open():
            return None
        return self.stop - self.start

    def to_slice(self):
        return slice(self.start, self.stop, 1)

    def __eq__(self, other):
        return self.start == other.start and self.stop == other.stop

    def __lt__(self, other):
        return self.start < other.start and self.to_closed(sys.maxint).stop < other.to_closed(sys.maxint).stop

    def __repr__(self):
        return "SequentialRange(start={0!r}, stop={1!r})".format(self.start, self.stop)

    def overlaps(self, other):
        a, b = (self, other) if self.start <= other.start else (other, self)
        a_imag_stop = a.to_closed(sys.maxint).stop
        return a_imag_stop > b.start

    def contains(self, point):
        return self.start <= point and (self.stop is None or self.stop > point)


class SequentialRangeList(list):
    def byte_length(self):
        sum = 0
        for r in self:
            if r.is_open():
                return None
            sum += r.length()
        return sum

    def has_overlaps(self):
        sorted_list = sorted(self)
        for i in xrange(0, len(sorted_list) - 1):
            if sorted_list[i].overlaps(sorted_list[i + 1]):
                return True
        return False

    def max_stop(self):
        m = 0
        for r in self:
            if r.is_open():
                return None
            m = max(m, r.stop)
        return m

    def sorted(self):
        return SequentialRangeList(sorted(self))

    def byte_offset(self, bytes):
        remaining_bytes = bytes
        for r in self:
            if r.is_open() or r.length() > remaining_bytes:
                return r.start + remaining_bytes
            else:
                remaining_bytes -= r.length()
        assert False, "requested byte offset {0!r} is outside the range list {1!r}".format(bytes, self)

    def is_open(self):
        return any(r.is_open() for r in self)

    def to_closed(self, total_len):
        return SequentialRangeList(r.to_closed(total_len) for r in self)

    def find_container(self, point):
        for r in self:
            if r.contains(point):
                return r
        return None

    def find_relative_container_index(self, point):
        sum_length = 0
        for i in xrange(len(self)):
            if self[i].is_open() or ((point - sum_length - self[i].length()) <= 0):
                return i, sum_length
            sum_length += self[i].length()
        return None

    def __add__(self, other):
        return SequentialRangeList(super(SequentialRangeList, self).__add__(other))

    def __radd__(self, other):
        return SequentialRangeList(super(SequentialRangeList, self).__radd__(other))


class RangeReference(Reference):
    def __add__(self, other):
        me = self.list if isinstance(self, ListRangeReference) else [self]
        if isinstance(other, ListRangeReference):
            return ListRangeReference(me + other.list)
        elif isinstance(other, RangeReference):
            return ListRangeReference(me + [other])
        return NotImplemented

    def __radd__(self, other):
        me = self.list if isinstance(self, ListRangeReference) else [self]
        if isinstance(other, ListRangeReference):
            return ListRangeReference(other.list + me)
        elif isinstance(other, Reference):
            return ListRangeReference([other] + me)
        return NotImplemented


class ListRangeReference(RangeReference):
    def __init__(self, l):
        super(ListRangeReference, self).__init__()
        self.list = [Reference.to_ref(obj) for obj in l]

    def evaluate(self, ctx):
        return SequentialRangeList(reduce(operator.add, [ref(ctx) for ref in self.list]))

    def __safe_repr__(self):
        return "[{0}]".format(", ".join(safe_repr(o for o in self.list)))


class BitContainer(object):
    def __init__(self):
        self.bits = BitRangeFactory(self)


class ByteListRangeReference(BitContainer, ListRangeReference):
    def __init__(self, l):
        BitContainer.__init__(self)
        ListRangeReference.__init__(self, l)


class ByteSliceRangeReference(BitContainer, RangeReference):
    def __init__(self, slic):
        assert slic.step in (1, None)
        assert slic.start is None or slic.start >= 0
        assert slic.stop is None or slic.stop >= 0
        assert (slic.start is None or slic.stop is None) or (slic.start <= slic.stop)

        BitContainer.__init__(self)
        RangeReference.__init__(self)

        self.start = Reference.to_ref(slic.start)
        self.stop = Reference.to_ref(slic.stop)

    def evaluate(self, ctx):
        return SequentialRangeList([SequentialRange(self.start(ctx), self.stop(ctx))])

    def __safe_repr__(self):
        return "[{0!r}:{1!r}]".format(safe_repr(self.start), safe_repr(self.stop))


class ByteNumericRangeReference(BitContainer, RangeReference):
    def __init__(self, ref):
        assert ref >= 0
        super(ByteNumericRangeReference, self).__init__()
        self.ref = Reference.to_ref(ref)

    def evaluate(self, ctx):
        index = self.ref(ctx)
        assert index >= 0
        return SequentialRangeList([SequentialRange(index, index + 1)])

    def __safe_repr__(self):
        return safe_repr(self.ref)


class ByteRangeFactory(object):
    def __getitem__(self, loc):
        if isinstance(loc, (tuple, list)):
            return ByteListRangeReference([self.__getitem__(loc_elem) for loc_elem in loc])
        if isinstance(loc, slice):
            return ByteSliceRangeReference(loc)
        if isinstance(loc, (NumericReference, int, long)):
            return ByteNumericRangeReference(loc)
        raise TypeError("{0} is not a supported index type".format(repr(loc)))


class BitRangeReference(RangeReference):
    def __init__(self, parent_range_ref):
        super(BitRangeReference, self).__init__()
        self.parent_range_ref = parent_range_ref


class BitNumericRangeReference(BitRangeReference):
    def __init__(self, parent_range_ref, ref):
        super(BitNumericRangeReference, self).__init__(parent_range_ref)
        self.ref = Reference.to_ref(ref)

    def evaluate(self, ctx):
        range_list = self.parent_range_ref(ctx)
        assert len(range_list) >= 1

        byte_offset = float(self.ref(ctx)) / 8
        assert byte_offset >= 0

        i, sum_length = range_list.find_relative_container_index(byte_offset)
        if i is None:
            raise ValueError("Bit offset {0} is out of range for range sequence {1!r}".format(self.ref(ctx),
                                                                                              range_list))
        return SequentialRangeList([SequentialRange(byte_offset - sum_length + range_list[i].start,
                                                    byte_offset - sum_length + range_list[i].start + BIT)])

    def __safe_repr__(self):
        return "{0}.bits[{1}]".format(safe_repr(self.parent_range_ref), safe_repr(self.ref))


class BitSliceRangeReference(BitRangeReference):
    def __init__(self, parent_range_ref, slic):
        assert slic.step in (1, None)
        assert slic.start is None or slic.start >= 0
        assert slic.stop is None or slic.stop >= 0
        assert (slic.start is None or slic.stop is None) or (slic.start <= slic.stop)

        super(BitSliceRangeReference, self).__init__(parent_range_ref)
        self.start = Reference.to_ref(slic.start)
        self.stop = Reference.to_ref(slic.stop)

    def evaluate(self, ctx):
        range_list = self.parent_range_ref(ctx)
        assert len(range_list) >= 1

        bit_range = SequentialRange(float(self.start(ctx)) / 8, float(self.stop(ctx)) / 8)

        i, sum_length = range_list.find_relative_container_index(bit_range.start)
        if i is None:
            raise ValueError("Bit offset {0} is out of range for range sequence {1!r}".format(
                int(bit_range.start * 8), range_list))

        if bit_range.is_open():
            subrange_start = range_list[i].start + bit_range.start - sum_length
            return SequentialRangeList([SequentialRange(subrange_start, range_list[i].stop)] + range_list[i + 1:])

        bit_range_remaining_len = bit_range.length()
        result = []
        for i in xrange(i, len(range_list)):
            r = range_list[i]
            subrange_start = r.start + bit_range.start - sum_length
            if r.is_open() or (r.stop - subrange_start) >= bit_range_remaining_len:
                subrange_stop = subrange_start + bit_range_remaining_len
            else:
                subrange_stop = r.stop

            result.append(SequentialRange(subrange_start, subrange_stop))

            bit_range_remaining_len -= (subrange_stop - subrange_start)
            if bit_range_remaining_len == 0:
                break

        if bit_range_remaining_len > 0:
            raise ValueError("Bit range {0!r} is out of range for parent range {1!r}".format(bit_range, range_list))

        return SequentialRangeList(result)

    def __safe_repr__(self):
        return "{0}.bits[{1}:{2}]".format(safe_repr(self.parent_range_ref), safe_repr(self.start), safe_repr(self.stop))


class BitRangeFactory(object):
    def __init__(self, parent_range_ref):
        self.parent_range_ref = parent_range_ref

    def __getitem__(self, loc):
        if isinstance(loc, (tuple, list)):
            return ListRangeReference([self.__getitem__(loc_elem) for loc_elem in loc])
        if isinstance(loc, slice):
            return BitSliceRangeReference(self.parent_range_ref, loc)
        if isinstance(loc, (NumericReference, int, long)):
            return BitNumericRangeReference(self.parent_range_ref, loc)
        raise TypeError("{0} is not a supported index type".format(repr(loc)))



from __future__ import absolute_import

import struct
import types
from infi.exceptools import chain

from .base import FixedSizer, Marshal, EMPTY_CONTEXT
from .errors import InstructError, NotEnoughDataError

class NumericMarshal(FixedSizer, Marshal):
    """
    Numeric fields template class. Using Python's ``struct`` package it serializes and deserializes numeric fields with
    different endianity and bit size.
    """
    def __init__(self, format_string, type):
        super(NumericMarshal, self).__init__()
        self.format_string = format_string
        self.type = type
        self.size = struct.calcsize(format_string)

    def create_from_stream(self, stream, context=EMPTY_CONTEXT):
        packed_value = stream.read(self.size)
        if len(packed_value) < self.size:
            raise NotEnoughDataError(expected=self.size, actually_read=len(packed_value))
        try:
            return struct.unpack(self.format_string, packed_value)[0]
        except struct.error, e:
            raise chain(InstructError("Unpacking error occurred"))

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        try:
            stream.write(struct.pack(self.format_string, obj))
        except struct.error, e:
            raise chain(InstructError("Packing error occurred"))

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        if self.type in (types.IntType, types.LongType):
            return context.get('int_repr_format', '%d') % obj
        elif self.type == types.FloatType:
            return context.get('float_repr_format', '%f') % obj
        return repr(obj)

"""unsigned, big endian 8-bit integer"""
UBInt8Marshal = NumericMarshal(">B", types.IntType)
"""unsigned, big endian 16-bit integer"""
UBInt16Marshal = NumericMarshal(">H", types.IntType)
"""unsigned, big endian 32-bit integer"""
UBInt32Marshal = NumericMarshal(">L", types.IntType)
"""unsigned, big endian 64-bit integer"""
UBInt64Marshal = NumericMarshal(">Q", types.LongType)
"""signed, big endian 8-bit integer"""
SBInt8Marshal = NumericMarshal(">b", types.IntType)
"""signed, big endian 16-bit integer"""
SBInt16Marshal = NumericMarshal(">h", types.IntType)
"""signed, big endian 32-bit integer"""
SBInt32Marshal = NumericMarshal(">l", types.IntType)
"""signed, big endian 64-bit integer"""
SBInt64Marshal = NumericMarshal(">q", types.IntType)
"""unsigned, little endian 8-bit integer"""
ULInt8Marshal = NumericMarshal("<B", types.IntType)
"""unsigned, little endian 16-bit integer"""
ULInt16Marshal = NumericMarshal("<H", types.IntType)
"""unsigned, little endian 32-bit integer"""
ULInt32Marshal = NumericMarshal("<L", types.IntType)
"""unsigned, little endian 64-bit integer"""
ULInt64Marshal = NumericMarshal("<Q", types.LongType)
"""signed, little endian 8-bit integer"""
SLInt8Marshal = NumericMarshal("<b", types.IntType)
"""signed, little endian 16-bit integer"""
SLInt16Marshal = NumericMarshal("<h", types.IntType)
"""signed, little endian 32-bit integer"""
SLInt32Marshal = NumericMarshal("<l", types.IntType)
"""signed, little endian 64-bit integer"""
SLInt64Marshal = NumericMarshal("<q", types.LongType)
"""unsigned, native endianity 8-bit integer"""
UNInt8Marshal = NumericMarshal("=B", types.IntType)
"""unsigned, native endianity 16-bit integer"""
UNInt16Marshal = NumericMarshal("=H", types.IntType)
"""unsigned, native endianity 32-bit integer"""
UNInt32Marshal = NumericMarshal("=L", types.IntType)
"""unsigned, native endianity 64-bit integer"""
UNInt64Marshal = NumericMarshal("=Q", types.LongType)
"""signed, native endianity 8-bit integer"""
SNInt8Marshal = NumericMarshal("=b", types.IntType)
"""signed, native endianity 16-bit integer"""
SNInt16Marshal = NumericMarshal("=h", types.IntType)
"""signed, native endianity 32-bit integer"""
SNInt32Marshal = NumericMarshal("=l", types.IntType)
"""signed, native endianity 64-bit integer"""
SNInt64Marshal = NumericMarshal("=q", types.LongType)
"""big endian, 32-bit IEEE floating point number"""
BFloat32Marshal = NumericMarshal(">f", types.FloatType)
"""little endian, 32-bit IEEE floating point number"""
LFloat32Marshal = NumericMarshal("<f", types.FloatType)
"""native endianity, 32-bit IEEE floating point number"""
NFloat32Marshal = NumericMarshal("=f", types.FloatType)
"""big endian, 64-bit IEEE floating point number"""
BFloat64Marshal = NumericMarshal(">d", types.FloatType)
"""little endian, 64-bit IEEE floating point number"""
LFloat64Marshal = NumericMarshal("<d", types.FloatType)
"""native endianity, 64-bit IEEE floating point number"""
NFloat64Marshal = NumericMarshal("=d", types.FloatType)



import math
import itertools

from infi import exceptools
from ..errors import InstructError

from .range import SequentialRangeList
from .reference import Context, Reference, NumericReference
from .io_buffer import InputBuffer, OutputBuffer


class InstructBufferError(InstructError):
    MESSAGE = """{error_msg} - attribute '{attr_name}' in class '{clazz}':
  Instruct internal call stack:
{context_call_stack}"""

    def __init__(self, error_msg, ctx, clazz, attr_name):
        # Format the context call stack, so it will be clearer.
        context_call_stack = "\n".join(["    {0}".format(line) for line in ctx.format_exception_call_stack()])
        msg = type(self).MESSAGE.format(error_msg=error_msg, attr_name=attr_name, clazz=clazz,
                                        context_call_stack=context_call_stack)
        super(InstructBufferError, self).__init__(msg)

        self.clazz = clazz
        self.attr_name = attr_name


class BufferContext(Context):
    """Base class for buffer context. Contains the object we're packing/unpacking and the list of fields."""

    def __init__(self, obj, fields):
        super(BufferContext, self).__init__()
        self.obj = obj
        self.fields = fields

    def is_pack(self):
        return isinstance(self, PackContext)

    def is_unpack(self):
        return isinstance(self, UnpackContext)


class PackContext(BufferContext):
    """Context used when packing. Contains the object, fields and output buffer."""

    def __init__(self, obj, fields):
        super(PackContext, self).__init__(obj, fields)
        self.output_buffer = OutputBuffer()


class UnpackContext(BufferContext):
    """Context used when unpacking. Contains the object, fields and input buffer."""

    def __init__(self, obj, fields, input_buffer):
        super(UnpackContext, self).__init__(obj, fields)
        self.input_buffer = InputBuffer(input_buffer)


class TotalSizeReference(Reference, NumericReference):
    def evaluate(self, ctx):
        # First, we'll try a shortcut - if the size is static, we'll return that since we precalculated it.
        size = getattr(type(ctx.obj), 'byte_size', None)
        if size is not None:
            return size

        if ctx.is_pack():
            lists = [field.pack_absolute_position_ref(ctx) for field in ctx.fields]
        else:
            lists = [field.unpack_absolute_position_ref(ctx) for field in ctx.fields]
        positions = SequentialRangeList(itertools.chain(*lists))
        result = positions.max_stop()  # total_size calculation
        assert result is not None
        return result

    def __safe_repr__(self):
        return "total_size"


class PackAbsolutePositionReference(Reference):
    def __init__(self, field, pack_position_ref):
        self.field = field
        self.pack_position_ref = pack_position_ref

    def is_open(self, ctx):
        return self.pack_position_ref(ctx).is_open()

    def evaluate(self, ctx):
        position_list = self.pack_position_ref(ctx)
        if position_list.has_overlaps():
            raise ValueError("field position list has overlapping ranges")

        if position_list.is_open():
            # We need the serialization result of this field to set the range. Note that we already checked if the
            # position has overlapping ranges, so there may be only a single open range.
            packed_field = self.field.pack_ref(ctx)
            current_length = 0
            absolute_position_list = []
            for pos in position_list:
                abs_pos = pos.to_closed(pos.start + len(packed_field) - current_length)
                absolute_position_list.append(abs_pos)
                current_length += abs_pos.length()
            return absolute_position_list
        else:
            return position_list

    def __safe_repr__(self):
        return "pack_abs_position({0!r}, {1!r})".format(self.field, self.position_list)


class UnpackAbsolutePositionReference(Reference):
    def __init__(self, field, unpack_position_ref):
        self.field = field
        self.unpack_position_ref = unpack_position_ref

    def is_open(self, ctx):
        return self.unpack_position_ref(ctx).is_open()

    def evaluate(self, ctx):
        position_list = self.unpack_position_ref(ctx)
        if position_list.has_overlaps():
            raise ValueError("field position list has overlapping ranges")

        if position_list.is_open():
            buffer_len = ctx.input_buffer.length()
            return position_list.to_closed(buffer_len)

        return position_list

    def __safe_repr__(self):
        return "unpack_abs_position(field={0!r}, {1!r})".format(self.field, self.unpack_position_ref)


class FieldReference(Reference):
    attr_name_ref = None
    pack_value_ref = None
    unpack_value_ref = None
    pack_ref = None
    unpack_ref = None
    pack_absolute_position_ref = None
    unpack_absolute_position_ref = None
    unpack_after = None
    default = None

    def init(self, name):
        self.attr_name_ref.obj = name

    def is_initialized(self):
        return self.attr_name_ref.obj is not None

    def attr_name(self):
        return self.attr_name_ref.obj

    def evaluate(self, ctx):
        if ctx.is_pack():
            return self.pack_value_ref(ctx)
        else:
            return self.unpack_value_ref(ctx)

    def __safe_repr__(self):
        return "field_ref({0!r})".format(self.attr_name_ref.obj)


class NumericFieldReference(FieldReference, NumericReference):
    def __safe_repr__(self):
        return "num_field_ref({0!r})".format(self.attr_name_ref.obj)


class BufferType(type):
    def __new__(cls, name, bases, attrs):
        # If we initialize our own class don't do any modifications.
        if name == "Buffer":
            return super(BufferType, cls).__new__(cls, name, bases, attrs)

        new_cls = super(BufferType, cls).__new__(cls, name, bases, attrs)

        # First off, assign names and getters/setters to all the field references and find all the fields.
        fields = []
        for attr_name in dir(new_cls):
            attr = getattr(new_cls, attr_name)
            if isinstance(attr, FieldReference) and not attr.is_initialized():
                attr.init(attr_name)
                fields.append(attr)

        setattr(new_cls, 'byte_size', attrs['byte_size'] if 'byte_size' in attrs else cls.calc_byte_size(name, fields))
        setattr(new_cls, '__fields__', fields)
        return new_cls

    @classmethod
    def calc_byte_size(cls, class_name, fields):
        ctx = PackContext(None, fields)
        positions = SequentialRangeList()
        for field in fields:  # we avoid list comprehension here so we'll know which field raised an error
            try:
                if not (field.pack_absolute_position_ref.is_static()
                        and field.unpack_absolute_position_ref.is_static()):
                    return None
                positions.extend(field.pack_absolute_position_ref(ctx))
            except:
                raise exceptools.chain(InstructBufferError("Error while calculating static byte size", ctx,
                                                           class_name, field.attr_name()))
        return positions.max_stop()


class Buffer(object):
    __metaclass__ = BufferType

    def __init__(self, **kwargs):
        super(Buffer, self).__init__()

        field_names = self._all_field_names()
        for name, value in kwargs.items():
            assert name in field_names, ("field {0} in class {1} is not defined but passed to Buffer's __init__"
                                         .format(name, type(self)))
            setattr(self, name, value)

        for field in self._all_fields():
            if isinstance(getattr(self, field.attr_name()), FieldReference):
                setattr(self, field.attr_name(), field.default)

    def pack(self):
        """Packs the object and returns a buffer representing the packed object."""
        fields = self._all_fields()
        ctx = PackContext(self, fields)

        for field in fields:
            try:
                ctx.output_buffer.set(field.pack_ref(ctx), field.pack_absolute_position_ref(ctx))
            except:
                raise exceptools.chain(InstructBufferError("Pack error occured", ctx, type(self), field.attr_name()))

        result = bytearray(ctx.output_buffer.get())

        # We want to support the user defining the buffer's fixed byte size but not using it all:
        static_byte_size = type(self).byte_size
        if static_byte_size:
            static_byte_size = int(math.ceil(static_byte_size))
            assert len(result) <= static_byte_size, \
                ("in type {0} computed pack size is {1} but declared byte size is {2} - perhaps you manually defined " +
                 "the byte size in the type but the actual size is bigger?").format(type(self), len(result),
                                                                                    static_byte_size)
            if len(result) < static_byte_size:
                result += bytearray(static_byte_size - len(result))
        return result

    def unpack(self, buffer):
        """Unpacks the object's fields from buffer."""
        fields = self._all_fields()
        ctx = UnpackContext(self, fields, buffer)

        for field in fields:
            try:
                for prev_field in field.unpack_after:
                    prev_field.unpack_value_ref(ctx)
                field.unpack_value_ref(ctx)
            except:
                raise exceptools.chain(InstructBufferError("Unpack error occurred", ctx, type(self), field.attr_name()))

        # Short-circuit: if we already have a static byte size, use that.
        if type(self).byte_size is not None:
            return type(self).byte_size
        return self.calc_byte_size()

    def calc_byte_size(self):
        """
        Returns this instance's size. If the size has to be calculated it may require packing some of the fields.
        """
        return TotalSizeReference()(PackContext(self, type(self).__fields__))

    def _all_fields(self):
        fields = []
        for cls in type(self).mro():
            cls_fields = getattr(cls, '__fields__', [])
            fields.extend(cls_fields)
        return fields

    def _all_field_names(self):
        return set(field.attr_name() for field in self._all_fields())

    def __repr__(self):
        fields = type(self).__fields__
        repr_fields = ["{0}={1!r}".format(field.attr_name(), getattr(self, field.attr_name())) for field in fields]
        return "{0}.{1}({2})".format(type(self).__module__, type(self).__name__, ", ".join(repr_fields))



def keep_kwargs_partial(func, *args, **keywords):
    """Like functools.partial but instead of using the new kwargs, keeps the old ones."""
    def newfunc(*fargs, **fkeywords):
        newkeywords = fkeywords.copy()
        newkeywords.update(keywords)
        return func(*(args + fargs), **newkeywords)
    newfunc.func = func
    newfunc.args = args
    newfunc.keywords = keywords
    return newfunc


def copy_and_remove_kwargs(kwargs, keys_to_remove):
    return dict((k, v) for k, v in kwargs.items() if k not in keys_to_remove)


def copy_defaults_and_override_with_kwargs(defaults, kwargs):
    d = defaults.copy()
    d.update(kwargs)
    return d


def copy_and_verify_fixed_kwargs(fixed_kwargs, kwargs):
    d = fixed_kwargs.copy()
    for k in fixed_kwargs.keys():
        assert k not in kwargs or kwargs[k] == d[k], "argument {0} must be {1} but instead is {2}".format(k, d[k], kwargs[k])
    d.update(kwargs)
    return d


def assert_kwarg_required(kwargs, key):
    assert key in kwargs, "{0} argument required but missing".format(key)


def assert_kwargs_not_conflicting(kwargs, key, conflicting_keys):
    if key in kwargs:
        assert not any(k in kwargs for k in conflicting_keys), \
            "{0} argument cannot be used with the following arguments: {1}".format(key, ", ".join(conflicting_keys))


def assert_kwarg_enum(kwargs, key, enum):
    if key in kwargs:
        assert kwargs[key] in enum, \
            "{0} argument must be one of: {1} but instead is {2}".format(key, ", ".join(enum), kwargs[key])


def assert_kwarg_exactly_single_out_of(kwargs, keys):
    keys_found = [k for k in keys if k in kwargs]
    if len(keys_found) == 0:
        assert False, "exactly one argument out of ({0}) is required but none were passed".format(", ".join(keys))
    else:
        assert len(keys_found) == 1, \
            "only one argument out of ({0}) is allowed but passed ({1})".format(", ".join(keys), ", ".join(keys_found))



import types

from .string import *
from .struct import Field

def PaddedString(name, size, padding="\x00", padding_direction=PADDING_DIRECTION_RIGHT, default=None):
    if default is not None and isinstance(size, types.IntType):
        assert len(default) <= size
    return Field(name, PaddedStringMarshal(size, padding, padding_direction), default)

def VarSizeString(name, size_marshal, padding="\x00", padding_direction=PADDING_DIRECTION_RIGHT, default=None):
    return Field(name, VarSizeStringMarshal(size_marshal, padding, padding_direction), default)

# Backward compatibility
FixedSizeString = PaddedString

def FixedSizeBuffer(name, size, default=None):
    if default is not None:
        assert len(default) == size
    return Field(name, FixedSizeBufferMarshal(size), default)

def VarSizeBuffer(name, size_marshal, default=None):
    return Field(name, VarSizeBufferMarshal(size_marshal), default)



from .numeric import *
from .base import Marshal
from .struct import Field

# Shortcut to allow users to use either UBInt8 as a serializer (e.g. in an array) or as a field (e.g. UBInt8("foo", 4))
class MarshalWithField(Marshal):
    def __init__(self, marshal):
        super(MarshalWithField, self).__init__()
        self.marshal = marshal

    def __call__(self, name, default=None):
        return Field(name, self.marshal, default)

    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        return self.marshal.create_from_stream(stream, context, *args, **kwargs)

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        return self.marshal.write_to_stream(obj, stream, context)

    def sizeof(self, obj):
        return self.marshal.sizeof(obj)

    def min_max_sizeof(self):
        return self.marshal.min_max_sizeof()

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return self.marshal.to_repr(obj, context)

__primitive_types__ = [ 'UBInt8', 'UBInt16', 'UBInt32', 'UBInt64',
                        'SBInt8', 'SBInt16', 'SBInt32', 'SBInt64',
                        'ULInt8', 'ULInt16', 'ULInt32', 'ULInt64',
                        'SLInt8', 'SLInt16', 'SLInt32', 'SLInt64',
                        'UNInt8', 'UNInt16', 'UNInt32', 'UNInt64',
                        'SNInt8', 'SNInt16', 'SNInt32', 'SNInt64',
                        'BFloat32', 'LFloat32', 'NFloat32', 'BFloat64', 'LFloat64', 'NFloat64' ]

for t in __primitive_types__:
    v = vars()
    v[t] = MarshalWithField(v["%sMarshal" % t])



from .base import Marshal, ConstReader, FixedSizer, EMPTY_CONTEXT, MinMax
from .errors import InstructError, InvalidValueError

PADDING_DIRECTION_NONE  = 0
PADDING_DIRECTION_RIGHT = 1
PADDING_DIRECTION_LEFT  = 2
PADDING_DIRECTION_BOTH  = 3

def _strip(obj, dir, padding):
    if dir == PADDING_DIRECTION_NONE:
        return obj
    elif dir == PADDING_DIRECTION_RIGHT:
        return obj.rstrip(padding)
    elif dir == PADDING_DIRECTION_LEFT:
        return obj.lstrip(padding)
    else:
        return obj.strip(padding)

def _pad(obj, size, dir, padding):
    pad_len = size - len(obj)
    if pad_len < 0:
        raise InvalidValueError("fixed-size item length is expected to be of length %d or smaller but instead got %d (item=%s)" % (size, len(obj), repr(obj)))
    elif pad_len == 0:
        return obj
    
    if dir == PADDING_DIRECTION_RIGHT or dir == PADDING_DIRECTION_BOTH:
        return obj + padding[0] * pad_len
    elif dir == PADDING_DIRECTION_LEFT:
        return (padding[0] * pad_len) + obj
    else:
        # PADDING_DIRECTION_NONE
        raise InvalidValueError("no padding specified but item length %d is smaller than required length %d (item=%s)" %
                                (len(obj), size, obj))

class VarSizeStringMarshal(Marshal):
    def __init__(self, size_marshal, padding='\x00', padding_direction=PADDING_DIRECTION_RIGHT):
        super(VarSizeStringMarshal, self).__init__()
        self.size_marshal = size_marshal
        self.padding = padding
        self.padding_direction = padding_direction

    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        buffer_len = self.size_marshal.create_from_stream(stream, context)
        obj = stream.read(buffer_len)
        if len(obj) < buffer_len:
            raise InstructError("Expected to read %d bytes from stream but read only %d" % (buffer_len, len(obj)))
        return _strip(obj, self.padding_direction, self.padding)

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        stripped_obj = _strip(obj, self.padding_direction, self.padding)
        self.size_marshal.write_to_stream(len(stripped_obj), stream, context)
        stream.write(stripped_obj)

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return repr(obj)

    def sizeof(self, obj):
        return self.size_marshal.sizeof(obj) + len(_strip(obj, self.padding_direction, self.padding))

    def min_max_sizeof(self):
        size_min_max = self.size_marshal.min_max_sizeof()
        return MinMax(size_min_max.min, size_min_max.max + (1 << (size_min_max.max * 8)) - 1)

class PaddedStringMarshal(FixedSizer, VarSizeStringMarshal):
    def __init__(self, size, padding='\x00', padding_direction=PADDING_DIRECTION_RIGHT):
        super(PaddedStringMarshal, self).__init__(ConstReader(size), padding, padding_direction)
        self.size = size

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        stripped_obj = _strip(obj, self.padding_direction, self.padding)
        stream.write(stripped_obj)
        if len(stripped_obj) < self.size:
            stream.write(self.padding[0] * (self.size - len(stripped_obj)))

class VarSizeBufferMarshal(VarSizeStringMarshal):
    def __init__(self, size_marshal):
        super(VarSizeBufferMarshal, self).__init__(size_marshal, padding='', padding_direction=PADDING_DIRECTION_NONE)

class FixedSizeBufferMarshal(PaddedStringMarshal):
    def __init__(self, size):
        super(FixedSizeBufferMarshal, self).__init__(size, padding='', padding_direction=PADDING_DIRECTION_NONE)



from . import Field
from ..base import EMPTY_CONTEXT
from ..errors import InstructError

class ConstField(Field):
    def __init__(self, name, marshal, value):
        super(ConstField, self).__init__(name, marshal)
        self.value = value

    def prepare_class(self, cls):
        setattr(cls, self.name, self)

    def prepare_instance(self, obj, args, kwargs):
        # Don't set the field from a kwarg.
        # TODO: take the key from the kwargs if exists and compare it to value?
        pass

    def read_fields(self, obj, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        value = self.marshal.create_from_stream(stream, context, *args, **kwargs)
        if value != self.value:
            raise InstructError("constant field %s of class %s should have the value %s but instead got %s" %
                                (self.name, type(obj), self.marshal.to_repr(self.value, context),
                                 self.marshal.to_repr(value, context)))

    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return "%s (const)" % super(ConstField, self).to_repr(obj, context)

    def get_value(self, obj):
        return self.value

    def set_value(self, obj, value):
        pass

    def __get__(self, obj, owner):
        return self.value

    def __set__(self, obj, value):
        if value != self.value:
            raise InstructError("trying to set a new value (%s) to const field %s of class %s" %
                                (value, self.name, type(obj)))




from .base import Marshal, ConstReader, EMPTY_CONTEXT, MinMax, FixedSizer

class ArrayBase(object):
    def to_repr(self, obj, context=EMPTY_CONTEXT):
        return "[ " + ", ".join([ self.element_marshal.to_repr(element, context) for element in obj ]) + " ]"

    def _approx_max_elements_by_size_bits(self):
        size_min_max = self.size_marshal.min_max_sizeof()
        return (1 << (size_min_max.max * 8)) - 1

    def min_max_sizeof(self):
        size_min_max = self.size_marshal.min_max_sizeof()
        element_min_max = self.element_marshal.min_max_sizeof()
        return MinMax(size_min_max.min,
                      size_min_max.max + self._approx_max_elements_by_size_bits() * element_min_max.max)

class VarSizeArrayMarshal(ArrayBase, Marshal):
    def __init__(self, size_marshal, element_marshal):
        super(VarSizeArrayMarshal, self).__init__()
        self.size_marshal = size_marshal
        self.element_marshal = element_marshal
    
    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        size = self.size_marshal.create_from_stream(stream, context, *args, **kwargs)
        obj = []
        for i in xrange(size):
            obj.append(self.element_marshal.create_from_stream(stream, context, *args, **kwargs))
        return obj

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        self.size_marshal.write_to_stream(len(obj), stream, context)
        for element in obj:
            self.element_marshal.write_to_stream(element, stream, context)

    def sizeof(self, obj):
        return self.size_marshal.sizeof(len(obj)) + sum([ self.element_marshal.sizeof(element) for element in obj ])

class FixedSizeArrayMarshal(VarSizeArrayMarshal):
    def __init__(self, size, element_marshal):
        super(FixedSizeArrayMarshal, self).__init__(ConstReader(size), element_marshal)
        self.size = size
    
    def min_max_sizeof(self):
        element_min_max = self.element_marshal.min_max_sizeof()
        return MinMax(element_min_max.min * self.size, element_min_max.max * self.size)

class SumSizeArrayMarshal(ArrayBase, Marshal):
    def __init__(self, size_marshal, element_marshal):
        super(SumSizeArrayMarshal, self).__init__()
        self.size_marshal = size_marshal
        self.element_marshal = element_marshal
    
    def create_from_stream(self, stream, context=EMPTY_CONTEXT, *args, **kwargs):
        obj = []
        total_bytes = self.size_marshal.create_from_stream(stream, context)
        has_tell = hasattr(stream, 'tell')
        while total_bytes > 0:
            element_size = stream.tell() if has_tell else 0
            element = self.element_marshal.create_from_stream(stream, context, *args, **kwargs)
            obj.append(element)
            element_size = stream.tell() - element_size if has_tell else self.element_marshal.sizeof(elemen)
            total_bytes -= element_size
        assert total_bytes == 0
        return obj

    def write_to_stream(self, obj, stream, context=EMPTY_CONTEXT):
        sum_size = sum([ self.element_marshal.sizeof(element) for element in obj ])
        self.size_marshal.write_to_stream(sum_size, stream, context)
        for element in obj:
            self.element_marshal.write_to_stream(element, stream, context)

    def sizeof(self, obj):
        sum_size = sum([ self.element_marshal.sizeof(element) for element in obj ])
        return self.size_marshal.sizeof(sum_size) + sum_size
